from __future__ import annotations

import os

from ..schemas.emotions import EMOTIONS, RESET, BOLD, DIM
from ..utils.emotion_lookup import ALL_EMOTIONS

ansi_color = lambda n: f"\033[38;5;{n}m"  # noqa: E731

ITALIC    = "\033[3m"
UNDERLINE = "\033[4m"

GOLD   = ansi_color(220)
SILVER = ansi_color(250)
GRAY   = ansi_color(240)
DGRAY  = ansi_color(236)

VALENCE_COLORS: dict[str, str] = {
    "positive": ansi_color(82),
    "negative": ansi_color(196),
    "mixed":    ansi_color(214),
}

INTENSITY_BARS = ["░", "▒", "▓", "█", "█"]

REGISTRATION_BANNERS = [
    "▸ awakened",
    "◆ initialized",
    "✦ activated",
    "⬡ registered",
    "⚙ configured",
    "◈ calibrated",
    "❋ initialized",
    "⊕ engaged",
]

BANNER_INDEX = 0


def next_banner() -> str:
    global BANNER_INDEX
    banner = REGISTRATION_BANNERS[BANNER_INDEX % len(REGISTRATION_BANNERS)]
    BANNER_INDEX += 1
    return banner


def terminal_width() -> int:
    try:
        return min(os.get_terminal_size().columns, 110)
    except OSError:
        return 90


def get_emotion_info(emotion: str) -> dict:
    return ALL_EMOTIONS.get(emotion, ALL_EMOTIONS.get("content", {
        "emoji": "🤖", "color": ansi_color(245), "family": "unknown",
        "intensity": 1, "valence": "mixed",
    }))


def intensity_bar(intensity: int) -> str:
    filled  = min(intensity, 5)
    char    = INTENSITY_BARS[filled - 1]
    colors  = [ansi_color(240), ansi_color(246), ansi_color(214), ansi_color(208), ansi_color(196)]
    color   = colors[filled - 1]
    return f"{color}{''.join(char * filled)}{RESET}{GRAY}{'·' * (5 - filled)}{RESET}"


def valence_badge(valence: str) -> str:
    symbols = {"positive": "▲ +", "negative": "▼ −", "mixed": "◆ ±"}
    colors  = {"positive": ansi_color(82), "negative": ansi_color(196), "mixed": ansi_color(214)}
    sym = symbols.get(valence, "◆ ?")
    col = colors.get(valence, ansi_color(245))
    return f"{col}{sym}{RESET}"


def say(emotion: str, func_name: str, message: str) -> None:
    info      = get_emotion_info(emotion)
    color     = info.get("color", ansi_color(245))
    emoji     = info.get("emoji", "🤖")
    family    = info.get("family", "unknown")
    intensity = info.get("intensity", 1)
    valence   = info.get("valence", "mixed")

    ibar  = intensity_bar(intensity)
    vbadge = valence_badge(valence)

    tag      = f"{color}{BOLD}{emoji} {func_name}{RESET}"
    family_s = f"{DIM}[{family}·{emotion}]{RESET}"
    right    = f"{ibar}  {vbadge}"

    print(f"  {tag}  {family_s}  {right}")
    print(f"  {GRAY}{'╌' * 4}{RESET}  {SILVER}{message}{RESET}")


def say_registration(emotion: str, func_name: str, message: str) -> None:
    info   = get_emotion_info(emotion)
    color  = info.get("color", ansi_color(245))
    emoji  = info.get("emoji", "🤖")
    family = info.get("family", "unknown")

    desc = message if len(message) <= 60 else message[:57] + "…"

    print(
        f"  {color}{BOLD}{emoji}  {func_name}{RESET}"
        f"  {GRAY}▸{RESET}  {color}{emotion}{RESET}"
        f"  {DIM}[{family}]{RESET}"
        f"  {SILVER}{ITALIC}{desc}{RESET}"
    )


def running(func_name: str) -> None:
    print(f"  {DGRAY}┄┄ {func_name} running ┄┄{RESET}")


def separator() -> None:
    print()


def say_evolution(
    func_name: str,
    old_emotion: str,
    new_emotion: str,
) -> None:
    old_info  = get_emotion_info(old_emotion)
    new_info  = get_emotion_info(new_emotion)
    old_color = old_info.get("color", ansi_color(245))
    new_color = new_info.get("color", ansi_color(245))
    old_emoji = old_info.get("emoji", "🤖")
    new_emoji = new_info.get("emoji", "🤖")

    print(
        f"  {GOLD}⟳ DRIFT{RESET}  "
        f"{old_color}{old_emoji} {func_name}{RESET}"
        f"  {GRAY}──▶{RESET}  "
        f"{new_color}{BOLD}{new_emoji} {new_emotion}{RESET}"
        f"  {GRAY}(emotion evolved){RESET}"
    )


def print_metrics_row(
    func_name: str,
    emotion: str,
    *,
    exception_raised: bool,
    call_count: int,
    wall_ms: float,
    cpu_ms: float,
    mem_delta_kb: float,
    mem_peak_kb: float,
    cyclomatic_complexity: int,
    exception_count: int,
    conversation_partner: str | None = None,
    conversation_partner_emotion: str | None = None,
) -> None:
    info  = get_emotion_info(emotion)
    color = info.get("color", ansi_color(245))
    emoji = info.get("emoji", "🤖")

    err_color  = ansi_color(196) if exception_count > 0 else ansi_color(240)
    avg_color  = ansi_color(196) if wall_ms > 500 else (ansi_color(214) if wall_ms > 100 else ansi_color(82))
    status_sym = "💥" if exception_raised else "✓"

    conv_part = ""
    if conversation_partner and conversation_partner_emotion:
        p_info  = get_emotion_info(conversation_partner_emotion)
        p_emoji = p_info.get("emoji", "🤖")
        p_color = p_info.get("color", ansi_color(245))
        conv_part = (
            f"  {GRAY}│{RESET}  💬 "
            f"{p_color}{p_emoji} {conversation_partner}{RESET}"
        )

    print(
        f"  {DGRAY}{status_sym}{RESET} "
        f"{color}{BOLD}{emoji} {func_name}{RESET}"
        f"  {GRAY}│{RESET}"
        f"  cpu={SILVER}{cpu_ms:.2f}ms{RESET}"
        f"  avg={avg_color}{wall_ms:.1f}ms{RESET}"
        f"  error={err_color}{exception_count}{RESET}"
        f"  {GRAY}│{RESET}  feeling {color}{emotion}{RESET}"
        f"{conv_part}"
    )



"""metrics.py — structured per-function report aggregating all data sources.

Combines:
  • 15-feature static vector        (core.analyzer)
  • Runtime counters                 (core.registry)
  • Profiling history                (core.registry profile_history)
  • Call graph                       (analysis.call_graph)
"""
from __future__ import annotations

from dataclasses import dataclass, field
from typing import Dict, List, Optional

from ..core import registry
from .call_graph import callers_of, callees_of, centrality, is_central, call_depth


@dataclass
class FunctionReport:
    # Identity
    name:        str
    emotion:     str
    description: str

    # ── Static metrics (from the 15-feature vector) ───────────────────────────
    branch_count:          int   = 0
    line_count:            int   = 0
    param_count:           int   = 0
    cyclomatic_complexity: float = 0.0
    nesting_depth:         float = 0.0
    return_count:          int   = 0
    has_recursion:         bool  = False
    has_async:             bool  = False
    has_try_except:        bool  = False
    has_yield:             bool  = False

    # ── Runtime metrics ────────────────────────────────────────────────────────
    call_count:       int   = 0
    exception_count:  int   = 0
    exception_rate:   float = 0.0   # fraction in [0.0, 1.0]
    average_time_ms:  float = 0.0
    total_time_ms:    float = 0.0

    # ── Profiling averages (over the last ≤ 50 calls) ─────────────────────────
    average_wall_time_ms:    float = 0.0
    average_cpu_time_ms:     float = 0.0
    average_memory_delta_kb: float = 0.0
    peak_memory_kb:          float = 0.0

    # ── Call graph ─────────────────────────────────────────────────────────────
    callers:          List[str] = field(default_factory=list)
    callees:          List[str] = field(default_factory=list)
    centrality_score: int       = 0
    is_central:       bool      = False
    max_call_depth:   int       = 0

    # ── Social relationships ───────────────────────────────────────────────────
    relationships: Dict[str, str] = field(default_factory=dict)

    # ── Derived helpers ────────────────────────────────────────────────────────

    def complexity_tier(self) -> str:
        complexity_value = self.cyclomatic_complexity
        if complexity_value <= 2:   return "trivial"
        if complexity_value <= 5:   return "simple"
        if complexity_value <= 10:  return "moderate"
        if complexity_value <= 20:  return "complex"
        return "very complex"

    def size_tier(self) -> str:
        line_count_value = self.line_count
        if line_count_value <= 5:   return "tiny"
        if line_count_value <= 20:  return "small"
        if line_count_value <= 60:  return "medium"
        if line_count_value <= 150: return "large"
        return "huge"

    def health_score(self) -> float:
        """Composite 0–100 health score (higher = healthier)."""
        score = 100.0
        if self.exception_rate > 0.5:    score -= 40
        elif self.exception_rate > 0.2:  score -= 20
        elif self.exception_rate > 0.05: score -= 8
        if self.cyclomatic_complexity > 20:   score -= 25
        elif self.cyclomatic_complexity > 10: score -= 12
        elif self.cyclomatic_complexity > 6:  score -= 5
        if self.average_time_ms > 2000:  score -= 20
        elif self.average_time_ms > 500: score -= 10
        elif self.average_time_ms > 100: score -= 3
        if self.peak_memory_kb > 51200:  score -= 15
        elif self.peak_memory_kb > 10240: score -= 8
        return max(0.0, round(score, 1))

    def star_rating(self) -> str:
        health_value = self.health_score()
        if health_value >= 90: return "⭐⭐⭐⭐⭐"
        if health_value >= 75: return "⭐⭐⭐⭐"
        if health_value >= 55: return "⭐⭐⭐"
        if health_value >= 35: return "⭐⭐"
        return "⭐"


# ── Builder functions ─────────────────────────────────────────────────────────

def build_report(name: str) -> Optional[FunctionReport]:
    """Build a FunctionReport from all available data for *name*."""
    function_data = registry.get_data(name)
    if not function_data:
        return None

    features        = function_data.get("features", [0.0] * 15)
    call_count      = function_data.get("call_count", 0)
    exception_count = function_data.get("exception_count", 0)
    exception_rate  = exception_count / call_count if call_count > 0 else 0.0
    profile_history = function_data.get("profile_history", [])

    average_wall_time_ms    = (
        sum(entry["wall_time_ms"]    for entry in profile_history) / len(profile_history)
        if profile_history else 0.0
    )
    average_cpu_time_ms     = (
        sum(entry["cpu_time_ms"]     for entry in profile_history) / len(profile_history)
        if profile_history else 0.0
    )
    average_memory_delta_kb = (
        sum(entry["memory_delta_kb"] for entry in profile_history) / len(profile_history)
        if profile_history else 0.0
    )
    peak_memory_kb          = max(
        (entry["memory_peak_kb"] for entry in profile_history), default=0.0
    )

    centrality_value = centrality(name)
    caller_names     = callers_of(name)
    callee_names     = callees_of(name)
    depth_value      = call_depth(name)

    return FunctionReport(
        name        = name,
        emotion     = function_data.get("emotion", "content"),
        description = function_data.get("description", ""),

        branch_count          = int(features[0]),
        line_count            = int(features[1]),
        param_count           = int(features[2]),
        has_recursion         = bool(features[3]),
        has_async             = bool(features[4]),
        has_try_except        = bool(features[5]),
        has_yield             = bool(features[6]),
        cyclomatic_complexity = round(features[7], 1),
        nesting_depth         = round(features[8], 1),
        return_count          = int(features[9]),

        call_count      = call_count,
        exception_count = exception_count,
        exception_rate  = round(exception_rate, 3),
        average_time_ms = round(function_data.get("average_time_ms", 0.0), 2),
        total_time_ms   = round(function_data.get("total_time_ms", 0.0), 2),

        average_wall_time_ms    = round(average_wall_time_ms, 2),
        average_cpu_time_ms     = round(average_cpu_time_ms, 3),
        average_memory_delta_kb = round(average_memory_delta_kb, 2),
        peak_memory_kb          = round(peak_memory_kb, 2),

        callers          = caller_names,
        callees          = callee_names,
        centrality_score = centrality_value,
        is_central       = is_central(name),
        max_call_depth   = depth_value,

        relationships = function_data.get("relationships", {}),
    )


def all_reports(sort_by: str = "call_count") -> List[FunctionReport]:
    """Return a FunctionReport for every registered function, sorted."""
    all_names = registry.get_all_names()
    reports   = [report for report in (build_report(name) for name in all_names)
                 if report is not None]
    if sort_by == "call_count":
        reports.sort(key=lambda report: report.call_count, reverse=True)
    elif sort_by == "health":
        reports.sort(key=lambda report: report.health_score(), reverse=True)
    elif sort_by == "complexity":
        reports.sort(key=lambda report: report.cyclomatic_complexity, reverse=True)
    elif sort_by == "name":
        reports.sort(key=lambda report: report.name)
    return reports



from __future__ import annotations
from ..schemas.emotions import EMOTIONS
from .ansi import RED, YELLOW, GREEN, CYAN, GRAY, ORANGE

try:
    from ..schemas.extended_emotions import EXTENDED_EMOTIONS
except ImportError:
    EXTENDED_EMOTIONS: dict = {}

ALL_EMOTIONS: dict = {**EMOTIONS, **EXTENDED_EMOTIONS}

RELATIONSHIP_COLORS: dict[str, str] = {
    "friendship":       GREEN,
    "anger":            RED,
    "jealousy":         YELLOW,
    "respect":          CYAN,
    "mock":             ORANGE,
    "pity":             GRAY,
    "confusion":        YELLOW,
    "class_respect":    CYAN,
    "rivalry":          RED,
    "dependency":       CYAN,
    "mentor":           GREEN,
    "sibling":          GRAY,
    "legacy_meets_new": ORANGE,
    "coworkers":        GRAY,
}



from __future__ import annotations

import functools
import inspect
import threading
from pathlib import Path
from typing import Callable, Optional

from . import analyzer, registry
from .profiler import Profiler
from ..analysis.call_graph import scan_directory
from ..analysis.narrator import describe
from ..output import logger

scan_lock = threading.Lock()
scan_done = False


def maybe_scan(func: Callable) -> None:
    global scan_done
    if scan_done:
        return
    with scan_lock:
        if scan_done:
            return
        try:
            source_file = inspect.getfile(func)
            scan_directory(Path(source_file).parent)
        except (TypeError, OSError):
            pass
        scan_done = True


active_calls_lock: threading.Lock = threading.Lock()
active_calls:      set[str]       = set()

conversation_funcs: set[str] = set()

conversation_queue_lock: threading.Lock                       = threading.Lock()
conversation_queue:      list[tuple[str, str, str, str, str]] = []
conversation_seen:       set[frozenset[str]]                  = set()


def flush_conversations() -> None:
    with conversation_queue_lock:
        items = list(conversation_queue)
        conversation_queue.clear()
    if not items:
        return
    try:
        from ..output.conversation import display_conversation
    except ImportError:
        return
    for func_a, emotion_a, func_b, emotion_b, relationship in items:
        display_conversation(func_a, emotion_a, func_b, emotion_b, relationship)


def emotion(
    decorated_func: Callable | None = None,
    *,
    enabled:        bool = True,
    conversation:   bool = False,
    verbose:        bool = True,
) -> Callable:
    def decorator(func: Callable) -> Callable:
        if not enabled:
            return func

        maybe_scan(func)

        is_method_flag = analyzer.is_method(func)
        features       = analyzer.extract_features(func)
        initial_emotion = analyzer.classify_emotion(features, func.__name__, is_method_flag)

        qualified_name: str           = getattr(func, "__qualname__", func.__name__)
        class_name:     Optional[str] = (
            qualified_name.rsplit(".", 1)[0] if "." in qualified_name else None
        )

        registry.register(func.__name__, initial_emotion, features, is_method_flag, class_name)

        description = describe(func)
        registry.set_description(func.__name__, description)

        if verbose:
            logger.say_registration(initial_emotion, func.__name__, description)

        if conversation:
            conversation_funcs.add(func.__name__)

        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            function_name = func.__name__

            with active_calls_lock:
                concurrent_names = list(active_calls)
                active_calls.add(function_name)
            registry.record_co_call(function_name, concurrent_names)

            conv_partner_name:    str | None = None
            conv_partner_emotion: str | None = None
            if conversation:
                conv_partners = [n for n in concurrent_names if n in conversation_funcs]
                if conv_partners:
                    conv_partner_name    = conv_partners[0]
                    conv_partner_emotion = registry.get_data(conv_partner_name).get(
                        "emotion", "content"
                    )

            call_start_time  = registry.record_call_start(function_name)
            exception_raised = False
            result           = None

            profiler = Profiler()
            profiler.__enter__()
            try:
                result = func(*args, **kwargs)
            except Exception:
                exception_raised = True
                raise
            finally:
                profiler.__exit__(None, None, None)
                profile_snapshot = profiler.snapshot

                with active_calls_lock:
                    active_calls.discard(function_name)

                registry.record_call_end(function_name, call_start_time,
                                         exception=exception_raised)

                if profile_snapshot is not None:
                    registry.record_profile(
                        function_name,
                        wall_time_ms    = profile_snapshot.wall_time_ms,
                        cpu_time_ms     = profile_snapshot.cpu_time_ms,
                        memory_delta_kb = profile_snapshot.memory_delta_kb,
                        memory_peak_kb  = profile_snapshot.memory_peak_kb,
                    )

                registry.update_relationships(function_name)

                current_emotion = registry.get_data(function_name).get("emotion", initial_emotion)
                if registry.should_evolve(function_name):
                    evolved_emotion = registry.compute_evolved_emotion(
                        function_name, current_emotion
                    )
                    if evolved_emotion != current_emotion:
                        registry.REGISTRY[function_name]["emotion"] = evolved_emotion
                        current_emotion = evolved_emotion

                if verbose:
                    function_data         = registry.get_data(function_name)
                    call_count            = function_data.get("call_count", 0)
                    exception_count       = function_data.get("exception_count", 0)
                    cyclomatic_complexity = int(features[7])

                    wall_ms = profile_snapshot.wall_time_ms if profile_snapshot else function_data.get("average_time_ms", 0.0)
                    cpu_ms  = profile_snapshot.cpu_time_ms  if profile_snapshot else 0.0
                    mem_d   = profile_snapshot.memory_delta_kb if profile_snapshot else 0.0
                    mem_p   = profile_snapshot.memory_peak_kb  if profile_snapshot else 0.0
                    logger.print_metrics_row(
                        function_name,
                        current_emotion,
                        exception_raised             = exception_raised,
                        call_count                   = call_count,
                        wall_ms                      = wall_ms,
                        cpu_ms                       = cpu_ms,
                        mem_delta_kb                 = mem_d,
                        mem_peak_kb                  = mem_p,
                        cyclomatic_complexity        = cyclomatic_complexity,
                        exception_count              = exception_count,
                        conversation_partner         = conv_partner_name,
                        conversation_partner_emotion = conv_partner_emotion,
                    )

                    if conv_partner_name and conv_partner_emotion:
                        queue_conversation(function_name, current_emotion,
                                           conv_partner_name, conv_partner_emotion)

            return result

        wrapper.__emotional__    = True
        wrapper.__emotion__      = initial_emotion
        wrapper.__description__  = description
        wrapper.__features__     = features
        wrapper.__conversation__ = conversation
        return wrapper

    if decorated_func is not None:
        return decorator(decorated_func)
    return decorator


def queue_conversation(
    func_a: str, emotion_a: str, func_b: str, emotion_b: str
) -> None:
    pair_key = frozenset([func_a, func_b])
    with conversation_queue_lock:
        if pair_key in conversation_seen:
            return
        conversation_seen.add(pair_key)
        relationship = best_relationship(func_a, func_b)
        conversation_queue.append((func_a, emotion_a, func_b, emotion_b, relationship))


def trigger_conversation(
    func_name: str,
    partners:  list[str],
) -> None:
    try:
        from ..output.conversation import display_conversation
    except ImportError:
        return

    partner_name = partners[0]
    emotion_a    = registry.get_data(func_name).get("emotion",       "content")
    emotion_b    = registry.get_data(partner_name).get("emotion",    "content")
    relationship = best_relationship(func_name, partner_name)

    display_conversation(func_name, emotion_a, partner_name, emotion_b, relationship)


def best_relationship(name_a: str, name_b: str) -> str | None:
    top = registry.get_top_relationships(name_a, max_results=5)
    for other_name, rel_type in top:
        if other_name == name_b:
            return rel_type
    return None



"""emotion — assigns emotional personalities to Python functions.

Quick start
-----------
    from emotion import emotion

    @emotion
    def process_data(records):
        return [r for r in records if r.valid]

    process_data([...])    # prints emotion + description + per-call metrics

    from emotion import show_dashboard
    show_dashboard()       # rich terminal report for all registered functions

Public API
----------
  emotion         — decorator: emotion, profiling, and logging
  show_dashboard  — terminal dashboard (health, metrics, call graph)
  build_report    — FunctionReport for a single registered function
  all_reports     — list of FunctionReport for every registered function
  describe        — one-sentence description of any callable
  callers_of      — static call graph: which functions call this one
  callees_of      — static call graph: which functions this one calls
  scan_directory  — manually trigger call-graph scanning on a directory
  get_all_names   — list of all registered function names
  get_data        — raw registry dict for a function
"""

from .core.decorator       import emotion, flush_conversations
from .output.dashboard     import show_dashboard
from .output.conversation  import display_conversation
from .analysis.metrics     import build_report, all_reports, FunctionReport
from .analysis.narrator    import describe
from .analysis.call_graph  import callers_of, callees_of, scan_directory
from .core.registry        import get_all_names, get_data

__all__ = [
    "emotion",
    "flush_conversations",
    "show_dashboard",
    "display_conversation",
    "build_report",
    "all_reports",
    "FunctionReport",
    "describe",
    "callers_of",
    "callees_of",
    "scan_directory",
    "get_all_names",
    "get_data",
]



"""500 human emotions — name, emoji, ANSI-256 color, family, intensity (1-5), valence."""

# ANSI 256-color format: \033[38;5;Nm
_C = lambda n: f"\033[38;5;{n}m"

EMOTIONS = {
    # ── JOY (20) ─────────────────────────────────────────────────────────────
    "content":        {"emoji": "😌", "color": _C(222), "name": "content",        "family": "joy", "intensity": 1, "valence": "positive"},
    "pleased":        {"emoji": "🙂", "color": _C(222), "name": "pleased",        "family": "joy", "intensity": 1, "valence": "positive"},
    "lighthearted":   {"emoji": "🌤️", "color": _C(226), "name": "lighthearted",   "family": "joy", "intensity": 1, "valence": "positive"},
    "glad":           {"emoji": "😊", "color": _C(226), "name": "glad",           "family": "joy", "intensity": 1, "valence": "positive"},
    "happy":          {"emoji": "😄", "color": _C(220), "name": "happy",          "family": "joy", "intensity": 2, "valence": "positive"},
    "cheerful":       {"emoji": "😁", "color": _C(220), "name": "cheerful",       "family": "joy", "intensity": 2, "valence": "positive"},
    "merry":          {"emoji": "🎊", "color": _C(220), "name": "merry",          "family": "joy", "intensity": 2, "valence": "positive"},
    "giddy":          {"emoji": "🌀", "color": _C(214), "name": "giddy",          "family": "joy", "intensity": 2, "valence": "positive"},
    "delighted":      {"emoji": "😆", "color": _C(214), "name": "delighted",      "family": "joy", "intensity": 3, "valence": "positive"},
    "gleeful":        {"emoji": "😸", "color": _C(214), "name": "gleeful",        "family": "joy", "intensity": 3, "valence": "positive"},
    "playful":        {"emoji": "🎮", "color": _C(190), "name": "playful",        "family": "joy", "intensity": 3, "valence": "positive"},
    "buoyant":        {"emoji": "🎈", "color": _C(190), "name": "buoyant",        "family": "joy", "intensity": 3, "valence": "positive"},
    "elated":         {"emoji": "🚀", "color": _C(154), "name": "elated",         "family": "joy", "intensity": 4, "valence": "positive"},
    "thrilled":       {"emoji": "⚡", "color": _C(154), "name": "thrilled",       "family": "joy", "intensity": 4, "valence": "positive"},
    "exuberant":      {"emoji": "🎉", "color": _C(118), "name": "exuberant",      "family": "joy", "intensity": 4, "valence": "positive"},
    "radiant":        {"emoji": "✨", "color": _C(118), "name": "radiant",        "family": "joy", "intensity": 4, "valence": "positive"},
    "joyful":         {"emoji": "😄", "color": _C(82),  "name": "joyful",         "family": "joy", "intensity": 5, "valence": "positive"},
    "ecstatic":       {"emoji": "🤩", "color": _C(82),  "name": "ecstatic",       "family": "joy", "intensity": 5, "valence": "positive"},
    "euphoric":       {"emoji": "🌟", "color": _C(46),  "name": "euphoric",       "family": "joy", "intensity": 5, "valence": "positive"},
    "jubilant":       {"emoji": "🏆", "color": _C(46),  "name": "jubilant",       "family": "joy", "intensity": 5, "valence": "positive"},

    # ── SERENITY (20) ────────────────────────────────────────────────────────
    "restful":        {"emoji": "💤", "color": _C(195), "name": "restful",        "family": "serenity", "intensity": 1, "valence": "positive"},
    "quiet":          {"emoji": "🤫", "color": _C(195), "name": "quiet",          "family": "serenity", "intensity": 1, "valence": "positive"},
    "still":          {"emoji": "🌙", "color": _C(189), "name": "still",          "family": "serenity", "intensity": 1, "valence": "positive"},
    "mellow":         {"emoji": "🎵", "color": _C(189), "name": "mellow",         "family": "serenity", "intensity": 1, "valence": "positive"},
    "calm":           {"emoji": "😮‍💨", "color": _C(159), "name": "calm",           "family": "serenity", "intensity": 2, "valence": "positive"},
    "relaxed":        {"emoji": "😴", "color": _C(159), "name": "relaxed",        "family": "serenity", "intensity": 2, "valence": "positive"},
    "comfortable":    {"emoji": "🛋️", "color": _C(123), "name": "comfortable",    "family": "serenity", "intensity": 2, "valence": "positive"},
    "cozy":           {"emoji": "🍵", "color": _C(123), "name": "cozy",           "family": "serenity", "intensity": 2, "valence": "positive"},
    "gentle":         {"emoji": "🌸", "color": _C(87),  "name": "gentle",         "family": "serenity", "intensity": 3, "valence": "positive"},
    "soft":           {"emoji": "🫧", "color": _C(87),  "name": "soft",           "family": "serenity", "intensity": 3, "valence": "positive"},
    "peaceful":       {"emoji": "☮️", "color": _C(51),  "name": "peaceful",       "family": "serenity", "intensity": 3, "valence": "positive"},
    "dreamy":         {"emoji": "💭", "color": _C(51),  "name": "dreamy",         "family": "serenity", "intensity": 3, "valence": "positive"},
    "serene":         {"emoji": "🌊", "color": _C(45),  "name": "serene",         "family": "serenity", "intensity": 4, "valence": "positive"},
    "tranquil":       {"emoji": "🕊️", "color": _C(45),  "name": "tranquil",       "family": "serenity", "intensity": 4, "valence": "positive"},
    "warm":           {"emoji": "🌻", "color": _C(39),  "name": "warm",           "family": "serenity", "intensity": 4, "valence": "positive"},
    "safe":           {"emoji": "🔒", "color": _C(39),  "name": "safe",           "family": "serenity", "intensity": 4, "valence": "positive"},
    "grounded":       {"emoji": "🌿", "color": _C(33),  "name": "grounded",       "family": "serenity", "intensity": 5, "valence": "positive"},
    "centered":       {"emoji": "⚖️", "color": _C(33),  "name": "centered",       "family": "serenity", "intensity": 5, "valence": "positive"},
    "composed":       {"emoji": "🧘", "color": _C(27),  "name": "composed",       "family": "serenity", "intensity": 5, "valence": "positive"},
    "languid":        {"emoji": "🌅", "color": _C(27),  "name": "languid",        "family": "serenity", "intensity": 5, "valence": "positive"},

    # ── GRATITUDE (10) ───────────────────────────────────────────────────────
    "lucky":          {"emoji": "🍀", "color": _C(120), "name": "lucky",          "family": "gratitude", "intensity": 1, "valence": "positive"},
    "thankful":       {"emoji": "💖", "color": _C(120), "name": "thankful",       "family": "gratitude", "intensity": 2, "valence": "positive"},
    "appreciative":   {"emoji": "🤲", "color": _C(119), "name": "appreciative",   "family": "gratitude", "intensity": 2, "valence": "positive"},
    "fortunate":      {"emoji": "⭐", "color": _C(119), "name": "fortunate",      "family": "gratitude", "intensity": 3, "valence": "positive"},
    "grateful":       {"emoji": "🙏", "color": _C(113), "name": "grateful",       "family": "gratitude", "intensity": 3, "valence": "positive"},
    "touched":        {"emoji": "🥹", "color": _C(113), "name": "touched",        "family": "gratitude", "intensity": 4, "valence": "positive"},
    "honored":        {"emoji": "🎗️", "color": _C(107), "name": "honored",        "family": "gratitude", "intensity": 4, "valence": "positive"},
    "blessed":        {"emoji": "🌈", "color": _C(107), "name": "blessed",        "family": "gratitude", "intensity": 4, "valence": "positive"},
    "privileged":     {"emoji": "👑", "color": _C(71),  "name": "privileged",     "family": "gratitude", "intensity": 5, "valence": "positive"},
    "moved":          {"emoji": "💫", "color": _C(71),  "name": "moved",          "family": "gratitude", "intensity": 5, "valence": "positive"},

    # ── LOVE (20) ────────────────────────────────────────────────────────────
    "kindhearted":    {"emoji": "😊", "color": _C(225), "name": "kindhearted",    "family": "love", "intensity": 1, "valence": "positive"},
    "caring":         {"emoji": "💗", "color": _C(225), "name": "caring",         "family": "love", "intensity": 1, "valence": "positive"},
    "generous":       {"emoji": "🎁", "color": _C(219), "name": "generous",       "family": "love", "intensity": 1, "valence": "positive"},
    "sympathetic":    {"emoji": "🤗", "color": _C(219), "name": "sympathetic",    "family": "love", "intensity": 1, "valence": "positive"},
    "empathetic":     {"emoji": "🫂", "color": _C(213), "name": "empathetic",     "family": "love", "intensity": 2, "valence": "positive"},
    "compassionate":  {"emoji": "💙", "color": _C(213), "name": "compassionate",  "family": "love", "intensity": 2, "valence": "positive"},
    "affectionate":   {"emoji": "🥰", "color": _C(207), "name": "affectionate",   "family": "love", "intensity": 2, "valence": "positive"},
    "nurturing":      {"emoji": "🤱", "color": _C(207), "name": "nurturing",      "family": "love", "intensity": 2, "valence": "positive"},
    "connected":      {"emoji": "🤝", "color": _C(201), "name": "connected",      "family": "love", "intensity": 3, "valence": "positive"},
    "bonded":         {"emoji": "🔗", "color": _C(201), "name": "bonded",         "family": "love", "intensity": 3, "valence": "positive"},
    "protective":     {"emoji": "🛡️", "color": _C(200), "name": "protective",     "family": "love", "intensity": 3, "valence": "positive"},
    "devoted":        {"emoji": "💕", "color": _C(200), "name": "devoted",        "family": "love", "intensity": 3, "valence": "positive"},
    "loving":         {"emoji": "❤️", "color": _C(198), "name": "loving",         "family": "love", "intensity": 4, "valence": "positive"},
    "intimate":       {"emoji": "🫶", "color": _C(198), "name": "intimate",       "family": "love", "intensity": 4, "valence": "positive"},
    "adoring":        {"emoji": "😍", "color": _C(197), "name": "adoring",        "family": "love", "intensity": 4, "valence": "positive"},
    "romantic":       {"emoji": "🌹", "color": _C(197), "name": "romantic",       "family": "love", "intensity": 4, "valence": "positive"},
    "passionate":     {"emoji": "🔥", "color": _C(196), "name": "passionate",     "family": "love", "intensity": 5, "valence": "positive"},
    "infatuated":     {"emoji": "💘", "color": _C(196), "name": "infatuated",     "family": "love", "intensity": 5, "valence": "positive"},
    "lovestruck":     {"emoji": "💞", "color": _C(204), "name": "lovestruck",     "family": "love", "intensity": 5, "valence": "positive"},
    "smitten":        {"emoji": "💓", "color": _C(204), "name": "smitten",        "family": "love", "intensity": 5, "valence": "positive"},

    # ── LONGING (10) ─────────────────────────────────────────────────────────
    "missing":        {"emoji": "🫙", "color": _C(183), "name": "missing",        "family": "longing", "intensity": 1, "valence": "mixed"},
    "reminiscent":    {"emoji": "🗓️", "color": _C(183), "name": "reminiscent",    "family": "longing", "intensity": 2, "valence": "mixed"},
    "sentimental":    {"emoji": "📷", "color": _C(177), "name": "sentimental",    "family": "longing", "intensity": 2, "valence": "mixed"},
    "nostalgic":      {"emoji": "🥹", "color": _C(177), "name": "nostalgic",      "family": "longing", "intensity": 3, "valence": "mixed"},
    "wistful":        {"emoji": "🌧️", "color": _C(171), "name": "wistful",        "family": "longing", "intensity": 3, "valence": "mixed"},
    "bittersweet":    {"emoji": "🍂", "color": _C(171), "name": "bittersweet",    "family": "longing", "intensity": 3, "valence": "mixed"},
    "pining":         {"emoji": "💌", "color": _C(165), "name": "pining",         "family": "longing", "intensity": 4, "valence": "negative"},
    "homesick":       {"emoji": "🏠", "color": _C(165), "name": "homesick",       "family": "longing", "intensity": 4, "valence": "negative"},
    "yearning":       {"emoji": "🌠", "color": _C(129), "name": "yearning",       "family": "longing", "intensity": 5, "valence": "negative"},
    "longing":        {"emoji": "💭", "color": _C(129), "name": "longing",        "family": "longing", "intensity": 5, "valence": "negative"},

    # ── SADNESS (20) ─────────────────────────────────────────────────────────
    "downcast":       {"emoji": "😓", "color": _C(153), "name": "downcast",       "family": "sadness", "intensity": 1, "valence": "negative"},
    "unhappy":        {"emoji": "😟", "color": _C(153), "name": "unhappy",        "family": "sadness", "intensity": 1, "valence": "negative"},
    "gloomy":         {"emoji": "🌩️", "color": _C(117), "name": "gloomy",         "family": "sadness", "intensity": 1, "valence": "negative"},
    "dejected":       {"emoji": "😔", "color": _C(117), "name": "dejected",       "family": "sadness", "intensity": 1, "valence": "negative"},
    "sad":            {"emoji": "😢", "color": _C(75),  "name": "sad",            "family": "sadness", "intensity": 2, "valence": "negative"},
    "sorrowful":      {"emoji": "😞", "color": _C(75),  "name": "sorrowful",      "family": "sadness", "intensity": 2, "valence": "negative"},
    "melancholic":    {"emoji": "🌧️", "color": _C(69),  "name": "melancholic",    "family": "sadness", "intensity": 2, "valence": "negative"},
    "forlorn":        {"emoji": "🥀", "color": _C(69),  "name": "forlorn",        "family": "sadness", "intensity": 2, "valence": "negative"},
    "dismal":         {"emoji": "⛈️", "color": _C(63),  "name": "dismal",         "family": "sadness", "intensity": 3, "valence": "negative"},
    "wretched":       {"emoji": "😿", "color": _C(63),  "name": "wretched",       "family": "sadness", "intensity": 3, "valence": "negative"},
    "miserable":      {"emoji": "😩", "color": _C(57),  "name": "miserable",      "family": "sadness", "intensity": 3, "valence": "negative"},
    "mournful":       {"emoji": "🪦", "color": _C(57),  "name": "mournful",       "family": "sadness", "intensity": 3, "valence": "negative"},
    "heartbroken":    {"emoji": "💔", "color": _C(27),  "name": "heartbroken",    "family": "sadness", "intensity": 4, "valence": "negative"},
    "desolate":       {"emoji": "🏚️", "color": _C(27),  "name": "desolate",       "family": "sadness", "intensity": 4, "valence": "negative"},
    "bereaved":       {"emoji": "🕯️", "color": _C(21),  "name": "bereaved",       "family": "sadness", "intensity": 4, "valence": "negative"},
    "lamenting":      {"emoji": "😫", "color": _C(21),  "name": "lamenting",      "family": "sadness", "intensity": 4, "valence": "negative"},
    "devastated":     {"emoji": "😭", "color": _C(18),  "name": "devastated",     "family": "sadness", "intensity": 5, "valence": "negative"},
    "despairing":     {"emoji": "🌑", "color": _C(18),  "name": "despairing",     "family": "sadness", "intensity": 5, "valence": "negative"},
    "inconsolable":   {"emoji": "💔", "color": _C(17),  "name": "inconsolable",   "family": "sadness", "intensity": 5, "valence": "negative"},
    "crushed":        {"emoji": "🫀", "color": _C(17),  "name": "crushed",        "family": "sadness", "intensity": 5, "valence": "negative"},

    # ── DISAPPOINTMENT (10) ──────────────────────────────────────────────────
    "deflated":       {"emoji": "🎈", "color": _C(246), "name": "deflated",       "family": "disappointment", "intensity": 1, "valence": "negative"},
    "disappointed":   {"emoji": "😞", "color": _C(246), "name": "disappointed",   "family": "disappointment", "intensity": 2, "valence": "negative"},
    "let_down":       {"emoji": "😔", "color": _C(243), "name": "let_down",       "family": "disappointment", "intensity": 2, "valence": "negative"},
    "crestfallen":    {"emoji": "😕", "color": _C(243), "name": "crestfallen",    "family": "disappointment", "intensity": 3, "valence": "negative"},
    "disillusioned":  {"emoji": "😒", "color": _C(240), "name": "disillusioned",  "family": "disappointment", "intensity": 3, "valence": "negative"},
    "regretful":      {"emoji": "😣", "color": _C(240), "name": "regretful",      "family": "disappointment", "intensity": 3, "valence": "negative"},
    "remorseful":     {"emoji": "😢", "color": _C(238), "name": "remorseful",     "family": "disappointment", "intensity": 4, "valence": "negative"},
    "disheartened":   {"emoji": "💔", "color": _C(238), "name": "disheartened",   "family": "disappointment", "intensity": 4, "valence": "negative"},
    "defeated":       {"emoji": "🏳️", "color": _C(236), "name": "defeated",       "family": "disappointment", "intensity": 5, "valence": "negative"},
    "resigned":       {"emoji": "😑", "color": _C(236), "name": "resigned",       "family": "disappointment", "intensity": 5, "valence": "negative"},

    # ── LONELINESS (10) ──────────────────────────────────────────────────────
    "overlooked":     {"emoji": "👤", "color": _C(61),  "name": "overlooked",     "family": "loneliness", "intensity": 1, "valence": "negative"},
    "excluded":       {"emoji": "🚫", "color": _C(61),  "name": "excluded",       "family": "loneliness", "intensity": 2, "valence": "negative"},
    "misunderstood":  {"emoji": "🤷", "color": _C(60),  "name": "misunderstood",  "family": "loneliness", "intensity": 2, "valence": "negative"},
    "neglected":      {"emoji": "🕳️", "color": _C(60),  "name": "neglected",      "family": "loneliness", "intensity": 3, "valence": "negative"},
    "lonely":         {"emoji": "🥺", "color": _C(55),  "name": "lonely",         "family": "loneliness", "intensity": 3, "valence": "negative"},
    "invisible":      {"emoji": "👻", "color": _C(55),  "name": "invisible",      "family": "loneliness", "intensity": 3, "valence": "negative"},
    "unwanted":       {"emoji": "❌", "color": _C(54),  "name": "unwanted",       "family": "loneliness", "intensity": 4, "valence": "negative"},
    "rejected":       {"emoji": "↩️", "color": _C(54),  "name": "rejected",       "family": "loneliness", "intensity": 4, "valence": "negative"},
    "abandoned":      {"emoji": "🚪", "color": _C(53),  "name": "abandoned",      "family": "loneliness", "intensity": 5, "valence": "negative"},
    "alienated":      {"emoji": "🌑", "color": _C(53),  "name": "alienated",      "family": "loneliness", "intensity": 5, "valence": "negative"},

    # ── FEAR (10) ────────────────────────────────────────────────────────────
    "wary":           {"emoji": "👀", "color": _C(209), "name": "wary",           "family": "fear", "intensity": 1, "valence": "negative"},
    "threatened":     {"emoji": "⚠️", "color": _C(209), "name": "threatened",     "family": "fear", "intensity": 2, "valence": "negative"},
    "fearful":        {"emoji": "😨", "color": _C(203), "name": "fearful",        "family": "fear", "intensity": 2, "valence": "negative"},
    "scared":         {"emoji": "😱", "color": _C(203), "name": "scared",         "family": "fear", "intensity": 3, "valence": "negative"},
    "dreading":       {"emoji": "😬", "color": _C(197), "name": "dreading",       "family": "fear", "intensity": 3, "valence": "negative"},
    "phobic":         {"emoji": "🕷️", "color": _C(197), "name": "phobic",         "family": "fear", "intensity": 4, "valence": "negative"},
    "paranoid":       {"emoji": "👁️", "color": _C(196), "name": "paranoid",       "family": "fear", "intensity": 4, "valence": "negative"},
    "petrified":      {"emoji": "🧊", "color": _C(196), "name": "petrified",      "family": "fear", "intensity": 5, "valence": "negative"},
    "horrified":      {"emoji": "😱", "color": _C(160), "name": "horrified",      "family": "fear", "intensity": 5, "valence": "negative"},
    "panicked":       {"emoji": "🆘", "color": _C(160), "name": "panicked",       "family": "fear", "intensity": 5, "valence": "negative"},

    # ── ANXIETY (10) ─────────────────────────────────────────────────────────
    "uneasy":         {"emoji": "😕", "color": _C(215), "name": "uneasy",         "family": "anxiety", "intensity": 1, "valence": "negative"},
    "nervous":        {"emoji": "😰", "color": _C(215), "name": "nervous",        "family": "anxiety", "intensity": 2, "valence": "negative"},
    "worried":        {"emoji": "😟", "color": _C(209), "name": "worried",        "family": "anxiety", "intensity": 2, "valence": "negative"},
    "apprehensive":   {"emoji": "😦", "color": _C(209), "name": "apprehensive",   "family": "anxiety", "intensity": 3, "valence": "negative"},
    "tense":          {"emoji": "😤", "color": _C(208), "name": "tense",          "family": "anxiety", "intensity": 3, "valence": "negative"},
    "stressed":       {"emoji": "🤯", "color": _C(208), "name": "stressed",       "family": "anxiety", "intensity": 3, "valence": "negative"},
    "jumpy":          {"emoji": "🐱", "color": _C(202), "name": "jumpy",          "family": "anxiety", "intensity": 4, "valence": "negative"},
    "on_edge":        {"emoji": "⚡", "color": _C(202), "name": "on_edge",        "family": "anxiety", "intensity": 4, "valence": "negative"},
    "anxious":        {"emoji": "😰", "color": _C(166), "name": "anxious",        "family": "anxiety", "intensity": 5, "valence": "negative"},
    "restless":       {"emoji": "🌀", "color": _C(166), "name": "restless",       "family": "anxiety", "intensity": 5, "valence": "negative"},

    # ── VULNERABILITY (10) ───────────────────────────────────────────────────
    "hesitant":       {"emoji": "⏸️", "color": _C(224), "name": "hesitant",       "family": "vulnerability", "intensity": 1, "valence": "negative"},
    "timid":          {"emoji": "🐭", "color": _C(224), "name": "timid",          "family": "vulnerability", "intensity": 1, "valence": "negative"},
    "cautious":       {"emoji": "🚧", "color": _C(218), "name": "cautious",       "family": "vulnerability", "intensity": 2, "valence": "neutral"},
    "doubtful":       {"emoji": "🤔", "color": _C(218), "name": "doubtful",       "family": "vulnerability", "intensity": 2, "valence": "negative"},
    "uncertain":      {"emoji": "❓", "color": _C(212), "name": "uncertain",      "family": "vulnerability", "intensity": 3, "valence": "negative"},
    "insecure":       {"emoji": "😟", "color": _C(212), "name": "insecure",       "family": "vulnerability", "intensity": 3, "valence": "negative"},
    "exposed":        {"emoji": "🫙", "color": _C(206), "name": "exposed",        "family": "vulnerability", "intensity": 4, "valence": "negative"},
    "fragile":        {"emoji": "🪞", "color": _C(206), "name": "fragile",        "family": "vulnerability", "intensity": 4, "valence": "negative"},
    "vulnerable":     {"emoji": "🫧", "color": _C(181), "name": "vulnerable",     "family": "vulnerability", "intensity": 5, "valence": "negative"},
    "raw":            {"emoji": "💗", "color": _C(181), "name": "raw",            "family": "vulnerability", "intensity": 5, "valence": "negative"},

    # ── ANGER (10) ───────────────────────────────────────────────────────────
    "irritable":      {"emoji": "😒", "color": _C(210), "name": "irritable",      "family": "anger", "intensity": 1, "valence": "negative"},
    "angry":          {"emoji": "😠", "color": _C(204), "name": "angry",          "family": "anger", "intensity": 2, "valence": "negative"},
    "irate":          {"emoji": "😾", "color": _C(204), "name": "irate",          "family": "anger", "intensity": 2, "valence": "negative"},
    "outraged":       {"emoji": "😤", "color": _C(197), "name": "outraged",       "family": "anger", "intensity": 3, "valence": "negative"},
    "incensed":       {"emoji": "🔥", "color": _C(197), "name": "incensed",       "family": "anger", "intensity": 3, "valence": "negative"},
    "furious":        {"emoji": "🤬", "color": _C(196), "name": "furious",        "family": "anger", "intensity": 4, "valence": "negative"},
    "livid":          {"emoji": "😡", "color": _C(196), "name": "livid",          "family": "anger", "intensity": 4, "valence": "negative"},
    "enraged":        {"emoji": "💢", "color": _C(160), "name": "enraged",        "family": "anger", "intensity": 5, "valence": "negative"},
    "wrathful":       {"emoji": "⚡", "color": _C(160), "name": "wrathful",       "family": "anger", "intensity": 5, "valence": "negative"},
    "seething":       {"emoji": "🌋", "color": _C(124), "name": "seething",       "family": "anger", "intensity": 5, "valence": "negative"},

    # ── IRRITATION (10) ──────────────────────────────────────────────────────
    "cranky":         {"emoji": "😾", "color": _C(216), "name": "cranky",         "family": "irritation", "intensity": 1, "valence": "negative"},
    "grumpy":         {"emoji": "😒", "color": _C(216), "name": "grumpy",         "family": "irritation", "intensity": 1, "valence": "negative"},
    "annoyed":        {"emoji": "😤", "color": _C(210), "name": "annoyed",        "family": "irritation", "intensity": 2, "valence": "negative"},
    "flustered":      {"emoji": "😅", "color": _C(210), "name": "flustered",      "family": "irritation", "intensity": 2, "valence": "negative"},
    "irritated":      {"emoji": "😒", "color": _C(172), "name": "irritated",      "family": "irritation", "intensity": 3, "valence": "negative"},
    "vexed":          {"emoji": "😫", "color": _C(172), "name": "vexed",          "family": "irritation", "intensity": 3, "valence": "negative"},
    "agitated":       {"emoji": "😤", "color": _C(166), "name": "agitated",       "family": "irritation", "intensity": 4, "valence": "negative"},
    "impatient":      {"emoji": "⏰", "color": _C(166), "name": "impatient",      "family": "irritation", "intensity": 4, "valence": "negative"},
    "frustrated":     {"emoji": "😣", "color": _C(130), "name": "frustrated",     "family": "irritation", "intensity": 5, "valence": "negative"},
    "exasperated":    {"emoji": "😩", "color": _C(130), "name": "exasperated",    "family": "irritation", "intensity": 5, "valence": "negative"},

    # ── RESENTMENT (10) ──────────────────────────────────────────────────────
    "offended":       {"emoji": "😤", "color": _C(173), "name": "offended",       "family": "resentment", "intensity": 1, "valence": "negative"},
    "indignant":      {"emoji": "😠", "color": _C(173), "name": "indignant",      "family": "resentment", "intensity": 2, "valence": "negative"},
    "bitter":         {"emoji": "🍋", "color": _C(136), "name": "bitter",         "family": "resentment", "intensity": 2, "valence": "negative"},
    "resentful":      {"emoji": "😒", "color": _C(136), "name": "resentful",      "family": "resentment", "intensity": 3, "valence": "negative"},
    "disdainful":     {"emoji": "😏", "color": _C(130), "name": "disdainful",     "family": "resentment", "intensity": 3, "valence": "negative"},
    "scornful":       {"emoji": "😤", "color": _C(130), "name": "scornful",       "family": "resentment", "intensity": 4, "valence": "negative"},
    "contemptuous":   {"emoji": "🙄", "color": _C(94),  "name": "contemptuous",   "family": "resentment", "intensity": 4, "valence": "negative"},
    "hostile":        {"emoji": "🗡️", "color": _C(94),  "name": "hostile",        "family": "resentment", "intensity": 4, "valence": "negative"},
    "spiteful":       {"emoji": "🐍", "color": _C(88),  "name": "spiteful",       "family": "resentment", "intensity": 5, "valence": "negative"},
    "vindictive":     {"emoji": "⚔️", "color": _C(88),  "name": "vindictive",     "family": "resentment", "intensity": 5, "valence": "negative"},

    # ── JEALOUSY (10) ────────────────────────────────────────────────────────
    "competitive":    {"emoji": "🏁", "color": _C(107), "name": "competitive",    "family": "jealousy", "intensity": 1, "valence": "neutral"},
    "envious":        {"emoji": "🟢", "color": _C(107), "name": "envious",        "family": "jealousy", "intensity": 2, "valence": "negative"},
    "covetous":       {"emoji": "👁️", "color": _C(70),  "name": "covetous",       "family": "jealousy", "intensity": 2, "valence": "negative"},
    "territorial":    {"emoji": "🦁", "color": _C(70),  "name": "territorial",    "family": "jealousy", "intensity": 3, "valence": "negative"},
    "jealous":        {"emoji": "😒", "color": _C(64),  "name": "jealous",        "family": "jealousy", "intensity": 3, "valence": "negative"},
    "overshadowed":   {"emoji": "🌑", "color": _C(64),  "name": "overshadowed",   "family": "jealousy", "intensity": 3, "valence": "negative"},
    "slighted":       {"emoji": "📉", "color": _C(58),  "name": "slighted",       "family": "jealousy", "intensity": 4, "valence": "negative"},
    "possessive":     {"emoji": "🔐", "color": _C(58),  "name": "possessive",     "family": "jealousy", "intensity": 4, "valence": "negative"},
    "inferior_feeling": {"emoji": "⬇️", "color": _C(22), "name": "inferior_feeling", "family": "jealousy", "intensity": 5, "valence": "negative"},
    "overlooked_deep": {"emoji": "👤", "color": _C(22), "name": "overlooked_deep", "family": "jealousy", "intensity": 5, "valence": "negative"},

    # ── DISGUST (10) ─────────────────────────────────────────────────────────
    "distasteful":    {"emoji": "😖", "color": _C(143), "name": "distasteful",    "family": "disgust", "intensity": 1, "valence": "negative"},
    "uncomfortable_d": {"emoji": "😣", "color": _C(143), "name": "uncomfortable_d", "family": "disgust", "intensity": 2, "valence": "negative"},
    "disgusted":      {"emoji": "🤢", "color": _C(100), "name": "disgusted",      "family": "disgust", "intensity": 2, "valence": "negative"},
    "repelled":       {"emoji": "↩️", "color": _C(100), "name": "repelled",       "family": "disgust", "intensity": 3, "valence": "negative"},
    "nauseated":      {"emoji": "🤢", "color": _C(58),  "name": "nauseated",      "family": "disgust", "intensity": 3, "valence": "negative"},
    "aversion_feeling": {"emoji": "🚫", "color": _C(58), "name": "aversion_feeling", "family": "disgust", "intensity": 3, "valence": "negative"},
    "repulsed":       {"emoji": "😷", "color": _C(94),  "name": "repulsed",       "family": "disgust", "intensity": 4, "valence": "negative"},
    "affronted":      {"emoji": "😤", "color": _C(94),  "name": "affronted",      "family": "disgust", "intensity": 4, "valence": "negative"},
    "revolted":       {"emoji": "🤮", "color": _C(88),  "name": "revolted",       "family": "disgust", "intensity": 5, "valence": "negative"},
    "appalled":       {"emoji": "😱", "color": _C(88),  "name": "appalled",       "family": "disgust", "intensity": 5, "valence": "negative"},

    # ── SURPRISE (10) ────────────────────────────────────────────────────────
    "startled":       {"emoji": "😮", "color": _C(159), "name": "startled",       "family": "surprise", "intensity": 1, "valence": "neutral"},
    "surprised":      {"emoji": "😲", "color": _C(87),  "name": "surprised",      "family": "surprise", "intensity": 2, "valence": "neutral"},
    "shocked":        {"emoji": "😳", "color": _C(87),  "name": "shocked",        "family": "surprise", "intensity": 2, "valence": "neutral"},
    "amazed":         {"emoji": "🤩", "color": _C(51),  "name": "amazed",         "family": "surprise", "intensity": 3, "valence": "positive"},
    "astonished":     {"emoji": "😮", "color": _C(51),  "name": "astonished",     "family": "surprise", "intensity": 3, "valence": "neutral"},
    "astounded":      {"emoji": "😯", "color": _C(45),  "name": "astounded",      "family": "surprise", "intensity": 4, "valence": "neutral"},
    "dumbfounded":    {"emoji": "🫢", "color": _C(45),  "name": "dumbfounded",    "family": "surprise", "intensity": 4, "valence": "neutral"},
    "flabbergasted":  {"emoji": "🙀", "color": _C(39),  "name": "flabbergasted",  "family": "surprise", "intensity": 5, "valence": "neutral"},
    "speechless":     {"emoji": "🤐", "color": _C(39),  "name": "speechless",     "family": "surprise", "intensity": 5, "valence": "neutral"},
    "stunned":        {"emoji": "💫", "color": _C(33),  "name": "stunned",        "family": "surprise", "intensity": 5, "valence": "neutral"},

    # ── WONDER (10) ──────────────────────────────────────────────────────────
    "curious":        {"emoji": "🧐", "color": _C(189), "name": "curious",        "family": "wonder", "intensity": 1, "valence": "positive"},
    "intrigued":      {"emoji": "🔍", "color": _C(147), "name": "intrigued",      "family": "wonder", "intensity": 2, "valence": "positive"},
    "fascinated":     {"emoji": "🤩", "color": _C(141), "name": "fascinated",     "family": "wonder", "intensity": 3, "valence": "positive"},
    "captivated":     {"emoji": "😍", "color": _C(135), "name": "captivated",     "family": "wonder", "intensity": 3, "valence": "positive"},
    "enchanted":      {"emoji": "✨", "color": _C(129), "name": "enchanted",      "family": "wonder", "intensity": 4, "valence": "positive"},
    "mesmerized":     {"emoji": "🌀", "color": _C(129), "name": "mesmerized",     "family": "wonder", "intensity": 4, "valence": "positive"},
    "spellbound":     {"emoji": "🪄", "color": _C(105), "name": "spellbound",     "family": "wonder", "intensity": 4, "valence": "positive"},
    "enthralled":     {"emoji": "🎭", "color": _C(105), "name": "enthralled",     "family": "wonder", "intensity": 5, "valence": "positive"},
    "awestruck":      {"emoji": "🌟", "color": _C(99),  "name": "awestruck",      "family": "wonder", "intensity": 5, "valence": "positive"},
    "wondering":      {"emoji": "🌌", "color": _C(99),  "name": "wondering",      "family": "wonder", "intensity": 5, "valence": "positive"},

    # ── ANTICIPATION (10) ────────────────────────────────────────────────────
    "expectant":      {"emoji": "👀", "color": _C(229), "name": "expectant",      "family": "anticipation", "intensity": 1, "valence": "positive"},
    "hopeful":        {"emoji": "🌱", "color": _C(229), "name": "hopeful",        "family": "anticipation", "intensity": 2, "valence": "positive"},
    "eager":          {"emoji": "🐕", "color": _C(226), "name": "eager",          "family": "anticipation", "intensity": 2, "valence": "positive"},
    "anticipating":   {"emoji": "⏳", "color": _C(220), "name": "anticipating",   "family": "anticipation", "intensity": 3, "valence": "positive"},
    "enthusiastic":   {"emoji": "🚀", "color": _C(220), "name": "enthusiastic",   "family": "anticipation", "intensity": 3, "valence": "positive"},
    "pumped":         {"emoji": "💪", "color": _C(214), "name": "pumped",         "family": "anticipation", "intensity": 4, "valence": "positive"},
    "zealous":        {"emoji": "⚡", "color": _C(214), "name": "zealous",        "family": "anticipation", "intensity": 4, "valence": "positive"},
    "excited":        {"emoji": "🎉", "color": _C(208), "name": "excited",        "family": "anticipation", "intensity": 4, "valence": "positive"},
    "fired_up":       {"emoji": "🔥", "color": _C(202), "name": "fired_up",       "family": "anticipation", "intensity": 5, "valence": "positive"},
    "buzzing":        {"emoji": "🐝", "color": _C(202), "name": "buzzing",        "family": "anticipation", "intensity": 5, "valence": "positive"},

    # ── CONFIDENCE (10) ──────────────────────────────────────────────────────
    "self_assured":   {"emoji": "😌", "color": _C(153), "name": "self_assured",   "family": "confidence", "intensity": 1, "valence": "positive"},
    "assertive":      {"emoji": "✊", "color": _C(117), "name": "assertive",      "family": "confidence", "intensity": 2, "valence": "positive"},
    "bold":           {"emoji": "💪", "color": _C(75),  "name": "bold",           "family": "confidence", "intensity": 2, "valence": "positive"},
    "confident":      {"emoji": "😎", "color": _C(75),  "name": "confident",      "family": "confidence", "intensity": 3, "valence": "positive"},
    "proud":          {"emoji": "🦁", "color": _C(33),  "name": "proud",          "family": "confidence", "intensity": 3, "valence": "positive"},
    "brave":          {"emoji": "🦸", "color": _C(33),  "name": "brave",          "family": "confidence", "intensity": 4, "valence": "positive"},
    "courageous":     {"emoji": "⚔️", "color": _C(27),  "name": "courageous",     "family": "confidence", "intensity": 4, "valence": "positive"},
    "daring":         {"emoji": "🪂", "color": _C(27),  "name": "daring",         "family": "confidence", "intensity": 5, "valence": "positive"},
    "fearless":       {"emoji": "🔱", "color": _C(21),  "name": "fearless",       "family": "confidence", "intensity": 5, "valence": "positive"},
    "invincible":     {"emoji": "🏆", "color": _C(21),  "name": "invincible",     "family": "confidence", "intensity": 5, "valence": "positive"},

    # ── AMBITION (10) ────────────────────────────────────────────────────────
    "focused":        {"emoji": "🎯", "color": _C(229), "name": "focused",        "family": "ambition", "intensity": 1, "valence": "positive"},
    "disciplined":    {"emoji": "📐", "color": _C(221), "name": "disciplined",    "family": "ambition", "intensity": 2, "valence": "positive"},
    "committed":      {"emoji": "🔒", "color": _C(214), "name": "committed",      "family": "ambition", "intensity": 2, "valence": "positive"},
    "motivated":      {"emoji": "🔥", "color": _C(208), "name": "motivated",      "family": "ambition", "intensity": 3, "valence": "positive"},
    "driven":         {"emoji": "💨", "color": _C(208), "name": "driven",         "family": "ambition", "intensity": 3, "valence": "positive"},
    "determined":     {"emoji": "🎯", "color": _C(202), "name": "determined",     "family": "ambition", "intensity": 4, "valence": "positive"},
    "dedicated":      {"emoji": "💎", "color": _C(202), "name": "dedicated",      "family": "ambition", "intensity": 4, "valence": "positive"},
    "persistent":     {"emoji": "🌊", "color": _C(196), "name": "persistent",     "family": "ambition", "intensity": 4, "valence": "positive"},
    "ambitious":      {"emoji": "🚀", "color": _C(196), "name": "ambitious",      "family": "ambition", "intensity": 5, "valence": "positive"},
    "relentless":     {"emoji": "⚡", "color": _C(160), "name": "relentless",     "family": "ambition", "intensity": 5, "valence": "positive"},

    # ── ACCOMPLISHMENT (10) ──────────────────────────────────────────────────
    "capable":        {"emoji": "💪", "color": _C(229), "name": "capable",        "family": "accomplishment", "intensity": 1, "valence": "positive"},
    "competent":      {"emoji": "🔧", "color": _C(221), "name": "competent",      "family": "accomplishment", "intensity": 2, "valence": "positive"},
    "satisfied":      {"emoji": "😌", "color": _C(221), "name": "satisfied",      "family": "accomplishment", "intensity": 2, "valence": "positive"},
    "gratified":      {"emoji": "😊", "color": _C(185), "name": "gratified",      "family": "accomplishment", "intensity": 3, "valence": "positive"},
    "fulfilled":      {"emoji": "💫", "color": _C(185), "name": "fulfilled",      "family": "accomplishment", "intensity": 3, "valence": "positive"},
    "accomplished":   {"emoji": "🏅", "color": _C(178), "name": "accomplished",   "family": "accomplishment", "intensity": 4, "valence": "positive"},
    "skilled":        {"emoji": "🎨", "color": _C(178), "name": "skilled",        "family": "accomplishment", "intensity": 4, "valence": "positive"},
    "successful":     {"emoji": "🌟", "color": _C(172), "name": "successful",     "family": "accomplishment", "intensity": 4, "valence": "positive"},
    "masterful":      {"emoji": "👑", "color": _C(136), "name": "masterful",      "family": "accomplishment", "intensity": 5, "valence": "positive"},
    "triumphant":     {"emoji": "🥇", "color": _C(136), "name": "triumphant",     "family": "accomplishment", "intensity": 5, "valence": "positive"},

    # ── HUMILITY (10) ────────────────────────────────────────────────────────
    "unassuming":     {"emoji": "🌱", "color": _C(253), "name": "unassuming",     "family": "humility", "intensity": 1, "valence": "positive"},
    "modest":         {"emoji": "🌿", "color": _C(250), "name": "modest",         "family": "humility", "intensity": 1, "valence": "positive"},
    "reserved":       {"emoji": "🚪", "color": _C(248), "name": "reserved",       "family": "humility", "intensity": 2, "valence": "neutral"},
    "bashful":        {"emoji": "🙈", "color": _C(248), "name": "bashful",        "family": "humility", "intensity": 2, "valence": "neutral"},
    "shy":            {"emoji": "😳", "color": _C(245), "name": "shy",            "family": "humility", "intensity": 3, "valence": "neutral"},
    "self_conscious": {"emoji": "😶", "color": _C(245), "name": "self_conscious", "family": "humility", "intensity": 3, "valence": "neutral"},
    "introverted":    {"emoji": "🔋", "color": _C(242), "name": "introverted",    "family": "humility", "intensity": 3, "valence": "neutral"},
    "meek":           {"emoji": "🕊️", "color": _C(242), "name": "meek",           "family": "humility", "intensity": 4, "valence": "neutral"},
    "humble":         {"emoji": "🙏", "color": _C(238), "name": "humble",         "family": "humility", "intensity": 4, "valence": "positive"},
    "diffident":      {"emoji": "🐢", "color": _C(238), "name": "diffident",      "family": "humility", "intensity": 5, "valence": "neutral"},

    # ── INTELLECTUAL (10) ────────────────────────────────────────────────────
    "systematic":     {"emoji": "📊", "color": _C(153), "name": "systematic",     "family": "intellectual", "intensity": 1, "valence": "neutral"},
    "methodical":     {"emoji": "📋", "color": _C(117), "name": "methodical",     "family": "intellectual", "intensity": 2, "valence": "neutral"},
    "logical":        {"emoji": "🧠", "color": _C(81),  "name": "logical",        "family": "intellectual", "intensity": 2, "valence": "neutral"},
    "analytical":     {"emoji": "🔬", "color": _C(75),  "name": "analytical",     "family": "intellectual", "intensity": 3, "valence": "neutral"},
    "precise":        {"emoji": "🎯", "color": _C(39),  "name": "precise",        "family": "intellectual", "intensity": 3, "valence": "neutral"},
    "strategic":      {"emoji": "♟️", "color": _C(39),  "name": "strategic",      "family": "intellectual", "intensity": 4, "valence": "neutral"},
    "thorough":       {"emoji": "📚", "color": _C(33),  "name": "thorough",       "family": "intellectual", "intensity": 4, "valence": "neutral"},
    "discerning":     {"emoji": "🔎", "color": _C(33),  "name": "discerning",     "family": "intellectual", "intensity": 4, "valence": "neutral"},
    "perfectionist":  {"emoji": "🔍", "color": _C(27),  "name": "perfectionist",  "family": "intellectual", "intensity": 5, "valence": "neutral"},
    "philosophical":  {"emoji": "💭", "color": _C(27),  "name": "philosophical",  "family": "intellectual", "intensity": 5, "valence": "neutral"},

    # ── CREATIVE (10) ────────────────────────────────────────────────────────
    "whimsical":      {"emoji": "🦋", "color": _C(219), "name": "whimsical",      "family": "creative", "intensity": 1, "valence": "positive"},
    "imaginative":    {"emoji": "🌈", "color": _C(213), "name": "imaginative",    "family": "creative", "intensity": 2, "valence": "positive"},
    "spontaneous":    {"emoji": "🌀", "color": _C(177), "name": "spontaneous",    "family": "creative", "intensity": 2, "valence": "positive"},
    "artistic":       {"emoji": "🖌️", "color": _C(141), "name": "artistic",       "family": "creative", "intensity": 3, "valence": "positive"},
    "expressive":     {"emoji": "🎭", "color": _C(141), "name": "expressive",     "family": "creative", "intensity": 3, "valence": "positive"},
    "inventive":      {"emoji": "🔧", "color": _C(135), "name": "inventive",      "family": "creative", "intensity": 4, "valence": "positive"},
    "innovative":     {"emoji": "🚀", "color": _C(135), "name": "innovative",     "family": "creative", "intensity": 4, "valence": "positive"},
    "inspired":       {"emoji": "💡", "color": _C(129), "name": "inspired",       "family": "creative", "intensity": 4, "valence": "positive"},
    "creative":       {"emoji": "🎨", "color": _C(93),  "name": "creative",       "family": "creative", "intensity": 5, "valence": "positive"},
    "visionary":      {"emoji": "🔭", "color": _C(93),  "name": "visionary",      "family": "creative", "intensity": 5, "valence": "positive"},

    # ── CONFUSION (10) ───────────────────────────────────────────────────────
    "muddled":        {"emoji": "🌫️", "color": _C(253), "name": "muddled",        "family": "confusion", "intensity": 1, "valence": "negative"},
    "puzzled":        {"emoji": "❓", "color": _C(250), "name": "puzzled",        "family": "confusion", "intensity": 2, "valence": "negative"},
    "perplexed":      {"emoji": "🤔", "color": _C(248), "name": "perplexed",      "family": "confusion", "intensity": 2, "valence": "negative"},
    "bewildered":     {"emoji": "😕", "color": _C(246), "name": "bewildered",     "family": "confusion", "intensity": 3, "valence": "negative"},
    "confused":       {"emoji": "😵", "color": _C(246), "name": "confused",       "family": "confusion", "intensity": 3, "valence": "negative"},
    "scattered":      {"emoji": "📌", "color": _C(244), "name": "scattered",      "family": "confusion", "intensity": 3, "valence": "negative"},
    "disoriented":    {"emoji": "💫", "color": _C(244), "name": "disoriented",    "family": "confusion", "intensity": 4, "valence": "negative"},
    "lost":           {"emoji": "🗺️", "color": _C(242), "name": "lost",           "family": "confusion", "intensity": 4, "valence": "negative"},
    "baffled":        {"emoji": "😵‍💫", "color": _C(242), "name": "baffled",        "family": "confusion", "intensity": 5, "valence": "negative"},
    "mystified":      {"emoji": "🌀", "color": _C(240), "name": "mystified",      "family": "confusion", "intensity": 5, "valence": "negative"},

    # ── INDECISION (10) ──────────────────────────────────────────────────────
    "wavering":       {"emoji": "🌊", "color": _C(253), "name": "wavering",       "family": "indecision", "intensity": 1, "valence": "negative"},
    "undecided":      {"emoji": "🎲", "color": _C(250), "name": "undecided",      "family": "indecision", "intensity": 2, "valence": "negative"},
    "ambivalent":     {"emoji": "🤷", "color": _C(248), "name": "ambivalent",     "family": "indecision", "intensity": 2, "valence": "neutral"},
    "torn":           {"emoji": "🫀", "color": _C(246), "name": "torn",           "family": "indecision", "intensity": 3, "valence": "negative"},
    "conflicted":     {"emoji": "⚡", "color": _C(244), "name": "conflicted",     "family": "indecision", "intensity": 3, "valence": "negative"},
    "vacillating":    {"emoji": "↔️", "color": _C(244), "name": "vacillating",    "family": "indecision", "intensity": 4, "valence": "negative"},
    "at_crossroads":  {"emoji": "🛤️", "color": _C(242), "name": "at_crossroads",  "family": "indecision", "intensity": 4, "valence": "neutral"},
    "indecisive":     {"emoji": "⚖️", "color": _C(242), "name": "indecisive",     "family": "indecision", "intensity": 4, "valence": "negative"},
    "irresolute":     {"emoji": "🌀", "color": _C(238), "name": "irresolute",     "family": "indecision", "intensity": 5, "valence": "negative"},
    "paralyzed":      {"emoji": "🧊", "color": _C(238), "name": "paralyzed",      "family": "indecision", "intensity": 5, "valence": "negative"},

    # ── BOREDOM (10) ─────────────────────────────────────────────────────────
    "stagnant":       {"emoji": "🏜️", "color": _C(244), "name": "stagnant",       "family": "boredom", "intensity": 1, "valence": "negative"},
    "monotonous":     {"emoji": "📏", "color": _C(242), "name": "monotonous",     "family": "boredom", "intensity": 1, "valence": "negative"},
    "indifferent":    {"emoji": "🪨", "color": _C(242), "name": "indifferent",    "family": "boredom", "intensity": 2, "valence": "neutral"},
    "disengaged":     {"emoji": "💤", "color": _C(240), "name": "disengaged",     "family": "boredom", "intensity": 2, "valence": "negative"},
    "listless":       {"emoji": "🍃", "color": _C(240), "name": "listless",       "family": "boredom", "intensity": 3, "valence": "negative"},
    "bored":          {"emoji": "😑", "color": _C(238), "name": "bored",          "family": "boredom", "intensity": 3, "valence": "negative"},
    "unmotivated":    {"emoji": "📉", "color": _C(238), "name": "unmotivated",    "family": "boredom", "intensity": 4, "valence": "negative"},
    "lethargic":      {"emoji": "🐌", "color": _C(236), "name": "lethargic",      "family": "boredom", "intensity": 4, "valence": "negative"},
    "sluggish":       {"emoji": "🐢", "color": _C(236), "name": "sluggish",       "family": "boredom", "intensity": 4, "valence": "negative"},
    "apathetic":      {"emoji": "😶", "color": _C(234), "name": "apathetic",      "family": "boredom", "intensity": 5, "valence": "negative"},

    # ── EXHAUSTION (10) ──────────────────────────────────────────────────────
    "tired":          {"emoji": "😴", "color": _C(244), "name": "tired",          "family": "exhaustion", "intensity": 1, "valence": "negative"},
    "weary":          {"emoji": "😪", "color": _C(242), "name": "weary",          "family": "exhaustion", "intensity": 2, "valence": "negative"},
    "fatigued":       {"emoji": "🥱", "color": _C(240), "name": "fatigued",       "family": "exhaustion", "intensity": 2, "valence": "negative"},
    "worn_out":       {"emoji": "👟", "color": _C(238), "name": "worn_out",       "family": "exhaustion", "intensity": 3, "valence": "negative"},
    "drained":        {"emoji": "🪫", "color": _C(238), "name": "drained",        "family": "exhaustion", "intensity": 3, "valence": "negative"},
    "depleted":       {"emoji": "⬇️", "color": _C(236), "name": "depleted",       "family": "exhaustion", "intensity": 4, "valence": "negative"},
    "spent":          {"emoji": "💤", "color": _C(236), "name": "spent",          "family": "exhaustion", "intensity": 4, "valence": "negative"},
    "exhausted":      {"emoji": "🔋", "color": _C(234), "name": "exhausted",      "family": "exhaustion", "intensity": 4, "valence": "negative"},
    "enervated":      {"emoji": "🌫️", "color": _C(234), "name": "enervated",      "family": "exhaustion", "intensity": 5, "valence": "negative"},
    "burnt_out":      {"emoji": "🕯️", "color": _C(232), "name": "burnt_out",      "family": "exhaustion", "intensity": 5, "valence": "negative"},

    # ── OVERWHELM (10) ───────────────────────────────────────────────────────
    "pressured":      {"emoji": "⚡", "color": _C(210), "name": "pressured",      "family": "overwhelm", "intensity": 1, "valence": "negative"},
    "burdened":       {"emoji": "⛏️", "color": _C(204), "name": "burdened",       "family": "overwhelm", "intensity": 2, "valence": "negative"},
    "swamped":        {"emoji": "🌊", "color": _C(204), "name": "swamped",        "family": "overwhelm", "intensity": 2, "valence": "negative"},
    "overloaded":     {"emoji": "📦", "color": _C(197), "name": "overloaded",     "family": "overwhelm", "intensity": 3, "valence": "negative"},
    "overwhelmed":    {"emoji": "🤯", "color": _C(197), "name": "overwhelmed",    "family": "overwhelm", "intensity": 3, "valence": "negative"},
    "smothered":      {"emoji": "🧱", "color": _C(196), "name": "smothered",      "family": "overwhelm", "intensity": 4, "valence": "negative"},
    "trapped":        {"emoji": "🕸️", "color": _C(160), "name": "trapped",        "family": "overwhelm", "intensity": 4, "valence": "negative"},
    "suffocated":     {"emoji": "💨", "color": _C(160), "name": "suffocated",     "family": "overwhelm", "intensity": 5, "valence": "negative"},
    "crushed_by_weight": {"emoji": "⬇️", "color": _C(124), "name": "crushed_by_weight", "family": "overwhelm", "intensity": 5, "valence": "negative"},
    "at_limit":       {"emoji": "🚨", "color": _C(124), "name": "at_limit",       "family": "overwhelm", "intensity": 5, "valence": "negative"},

    # ── EMPOWERMENT (10) ─────────────────────────────────────────────────────
    "independent":    {"emoji": "🔓", "color": _C(157), "name": "independent",    "family": "empowerment", "intensity": 1, "valence": "positive"},
    "self_reliant":   {"emoji": "🏔️", "color": _C(121), "name": "self_reliant",   "family": "empowerment", "intensity": 2, "valence": "positive"},
    "autonomous":     {"emoji": "🤖", "color": _C(121), "name": "autonomous",     "family": "empowerment", "intensity": 2, "valence": "positive"},
    "empowered":      {"emoji": "⚡", "color": _C(82),  "name": "empowered",      "family": "empowerment", "intensity": 3, "valence": "positive"},
    "liberated":      {"emoji": "🦅", "color": _C(82),  "name": "liberated",      "family": "empowerment", "intensity": 3, "valence": "positive"},
    "sovereign":      {"emoji": "👑", "color": _C(46),  "name": "sovereign",      "family": "empowerment", "intensity": 4, "valence": "positive"},
    "untethered":     {"emoji": "🪁", "color": _C(46),  "name": "untethered",     "family": "empowerment", "intensity": 4, "valence": "positive"},
    "free":           {"emoji": "🌬️", "color": _C(48),  "name": "free",           "family": "empowerment", "intensity": 4, "valence": "positive"},
    "unrestricted":   {"emoji": "🚀", "color": _C(48),  "name": "unrestricted",   "family": "empowerment", "intensity": 5, "valence": "positive"},
    "limitless":      {"emoji": "♾️", "color": _C(40),  "name": "limitless",      "family": "empowerment", "intensity": 5, "valence": "positive"},

    # ── BELONGING (10) ───────────────────────────────────────────────────────
    "included":       {"emoji": "🤗", "color": _C(157), "name": "included",       "family": "belonging", "intensity": 1, "valence": "positive"},
    "accepted":       {"emoji": "✅", "color": _C(121), "name": "accepted",       "family": "belonging", "intensity": 2, "valence": "positive"},
    "understood":     {"emoji": "💡", "color": _C(85),  "name": "understood",     "family": "belonging", "intensity": 2, "valence": "positive"},
    "seen":           {"emoji": "👁️", "color": _C(85),  "name": "seen",           "family": "belonging", "intensity": 3, "valence": "positive"},
    "valued":         {"emoji": "💎", "color": _C(78),  "name": "valued",         "family": "belonging", "intensity": 3, "valence": "positive"},
    "respected":      {"emoji": "🫡", "color": _C(78),  "name": "respected",      "family": "belonging", "intensity": 3, "valence": "positive"},
    "admired":        {"emoji": "⭐", "color": _C(72),  "name": "admired",        "family": "belonging", "intensity": 4, "valence": "positive"},
    "celebrated":     {"emoji": "🎊", "color": _C(72),  "name": "celebrated",     "family": "belonging", "intensity": 4, "valence": "positive"},
    "cherished":      {"emoji": "💖", "color": _C(36),  "name": "cherished",      "family": "belonging", "intensity": 5, "valence": "positive"},
    "belonging":      {"emoji": "🏡", "color": _C(36),  "name": "belonging",      "family": "belonging", "intensity": 5, "valence": "positive"},

    # ── BETRAYAL (10) ────────────────────────────────────────────────────────
    "guarded":        {"emoji": "🛡️", "color": _C(138), "name": "guarded",        "family": "betrayal", "intensity": 1, "valence": "negative"},
    "distrustful":    {"emoji": "🔐", "color": _C(131), "name": "distrustful",    "family": "betrayal", "intensity": 2, "valence": "negative"},
    "closed_off":     {"emoji": "🚪", "color": _C(131), "name": "closed_off",     "family": "betrayal", "intensity": 2, "valence": "negative"},
    "cynical":        {"emoji": "🔩", "color": _C(94),  "name": "cynical",        "family": "betrayal", "intensity": 3, "valence": "negative"},
    "jaded":          {"emoji": "😒", "color": _C(94),  "name": "jaded",          "family": "betrayal", "intensity": 3, "valence": "negative"},
    "used":           {"emoji": "🪤", "color": _C(88),  "name": "used",           "family": "betrayal", "intensity": 4, "valence": "negative"},
    "manipulated":    {"emoji": "🕹️", "color": _C(88),  "name": "manipulated",    "family": "betrayal", "intensity": 4, "valence": "negative"},
    "deceived":       {"emoji": "🎭", "color": _C(52),  "name": "deceived",       "family": "betrayal", "intensity": 4, "valence": "negative"},
    "betrayed":       {"emoji": "🗡️", "color": _C(52),  "name": "betrayed",       "family": "betrayal", "intensity": 5, "valence": "negative"},
    "taken_for_granted": {"emoji": "🚮", "color": _C(52), "name": "taken_for_granted", "family": "betrayal", "intensity": 5, "valence": "negative"},

    # ── MORAL (10) ───────────────────────────────────────────────────────────
    "fair":           {"emoji": "⚖️", "color": _C(229), "name": "fair",           "family": "moral", "intensity": 1, "valence": "positive"},
    "ethical":        {"emoji": "🧭", "color": _C(229), "name": "ethical",        "family": "moral", "intensity": 2, "valence": "positive"},
    "principled":     {"emoji": "📜", "color": _C(228), "name": "principled",     "family": "moral", "intensity": 2, "valence": "positive"},
    "honorable":      {"emoji": "🎖️", "color": _C(228), "name": "honorable",      "family": "moral", "intensity": 3, "valence": "positive"},
    "dignified":      {"emoji": "👔", "color": _C(222), "name": "dignified",      "family": "moral", "intensity": 3, "valence": "positive"},
    "just":           {"emoji": "🏛️", "color": _C(222), "name": "just",           "family": "moral", "intensity": 3, "valence": "positive"},
    "virtuous":       {"emoji": "🌟", "color": _C(186), "name": "virtuous",       "family": "moral", "intensity": 4, "valence": "positive"},
    "forgiving":      {"emoji": "🕊️", "color": _C(186), "name": "forgiving",      "family": "moral", "intensity": 4, "valence": "positive"},
    "merciful":       {"emoji": "💙", "color": _C(150), "name": "merciful",       "family": "moral", "intensity": 4, "valence": "positive"},
    "righteous":      {"emoji": "⚖️", "color": _C(150), "name": "righteous",      "family": "moral", "intensity": 5, "valence": "positive"},

    # ── SHAME_DEEP (10) ──────────────────────────────────────────────────────
    "self_critical":  {"emoji": "🔍", "color": _C(244), "name": "self_critical",  "family": "shame_deep", "intensity": 1, "valence": "negative"},
    "embarrassed":    {"emoji": "😖", "color": _C(242), "name": "embarrassed",    "family": "shame_deep", "intensity": 2, "valence": "negative"},
    "ashamed":        {"emoji": "😔", "color": _C(240), "name": "ashamed",        "family": "shame_deep", "intensity": 2, "valence": "negative"},
    "guilty":         {"emoji": "😢", "color": _C(238), "name": "guilty",         "family": "shame_deep", "intensity": 3, "valence": "negative"},
    "shamed":         {"emoji": "🙈", "color": _C(238), "name": "shamed",         "family": "shame_deep", "intensity": 3, "valence": "negative"},
    "mortified":      {"emoji": "😱", "color": _C(237), "name": "mortified",      "family": "shame_deep", "intensity": 4, "valence": "negative"},
    "humiliated":     {"emoji": "😤", "color": _C(237), "name": "humiliated",     "family": "shame_deep", "intensity": 4, "valence": "negative"},
    "degraded":       {"emoji": "⬇️", "color": _C(236), "name": "degraded",       "family": "shame_deep", "intensity": 4, "valence": "negative"},
    "worthless":      {"emoji": "🗑️", "color": _C(234), "name": "worthless",      "family": "shame_deep", "intensity": 5, "valence": "negative"},
    "self_loathing":  {"emoji": "💔", "color": _C(234), "name": "self_loathing",  "family": "shame_deep", "intensity": 5, "valence": "negative"},

    # ── EXISTENTIAL (10) ─────────────────────────────────────────────────────
    "naive":          {"emoji": "🌱", "color": _C(183), "name": "naive",          "family": "existential", "intensity": 1, "valence": "neutral"},
    "innocent":       {"emoji": "🕊️", "color": _C(183), "name": "innocent",       "family": "existential", "intensity": 1, "valence": "positive"},
    "purposeful":     {"emoji": "🎯", "color": _C(105), "name": "purposeful",     "family": "existential", "intensity": 2, "valence": "positive"},
    "meaningful":     {"emoji": "✨", "color": _C(105), "name": "meaningful",     "family": "existential", "intensity": 3, "valence": "positive"},
    "wise":           {"emoji": "🦉", "color": _C(99),  "name": "wise",           "family": "existential", "intensity": 3, "valence": "positive"},
    "enlightened":    {"emoji": "💡", "color": _C(99),  "name": "enlightened",    "family": "existential", "intensity": 4, "valence": "positive"},
    "existential":    {"emoji": "🌌", "color": _C(55),  "name": "existential",    "family": "existential", "intensity": 4, "valence": "neutral"},
    "purposeless":    {"emoji": "🌫️", "color": _C(55),  "name": "purposeless",    "family": "existential", "intensity": 4, "valence": "negative"},
    "meaningless":    {"emoji": "🕳️", "color": _C(54),  "name": "meaningless",    "family": "existential", "intensity": 5, "valence": "negative"},
    "transcendent":   {"emoji": "🌠", "color": _C(54),  "name": "transcendent",   "family": "existential", "intensity": 5, "valence": "positive"},

    # ── POWER (10) ───────────────────────────────────────────────────────────
    "capable_power":  {"emoji": "💪", "color": _C(210), "name": "capable_power",  "family": "power", "intensity": 1, "valence": "positive"},
    "in_control":     {"emoji": "🎮", "color": _C(204), "name": "in_control",     "family": "power", "intensity": 2, "valence": "positive"},
    "authoritative":  {"emoji": "📢", "color": _C(197), "name": "authoritative",  "family": "power", "intensity": 3, "valence": "neutral"},
    "commanding":     {"emoji": "⚔️", "color": _C(197), "name": "commanding",     "family": "power", "intensity": 3, "valence": "neutral"},
    "dominant":       {"emoji": "🦁", "color": _C(196), "name": "dominant",       "family": "power", "intensity": 4, "valence": "neutral"},
    "powerful":       {"emoji": "⚡", "color": _C(196), "name": "powerful",       "family": "power", "intensity": 4, "valence": "positive"},
    "helpless":       {"emoji": "🙌", "color": _C(240), "name": "helpless",       "family": "power", "intensity": 3, "valence": "negative"},
    "powerless":      {"emoji": "🔋", "color": _C(238), "name": "powerless",      "family": "power", "intensity": 4, "valence": "negative"},
    "submissive":     {"emoji": "🌊", "color": _C(236), "name": "submissive",     "family": "power", "intensity": 4, "valence": "negative"},
    "controlled":     {"emoji": "🕹️", "color": _C(234), "name": "controlled",     "family": "power", "intensity": 5, "valence": "negative"},

    # ── HUMOR (10) ───────────────────────────────────────────────────────────
    "amused":         {"emoji": "😄", "color": _C(190), "name": "amused",         "family": "humor", "intensity": 1, "valence": "positive"},
    "entertained":    {"emoji": "🎭", "color": _C(190), "name": "entertained",    "family": "humor", "intensity": 2, "valence": "positive"},
    "witty":          {"emoji": "🃏", "color": _C(154), "name": "witty",          "family": "humor", "intensity": 2, "valence": "positive"},
    "playful_humor":  {"emoji": "😜", "color": _C(154), "name": "playful_humor",  "family": "humor", "intensity": 3, "valence": "positive"},
    "mischievous":    {"emoji": "😈", "color": _C(118), "name": "mischievous",    "family": "humor", "intensity": 3, "valence": "mixed"},
    "cheeky":         {"emoji": "😋", "color": _C(118), "name": "cheeky",         "family": "humor", "intensity": 3, "valence": "positive"},
    "sarcastic":      {"emoji": "🙄", "color": _C(148), "name": "sarcastic",      "family": "humor", "intensity": 4, "valence": "mixed"},
    "ironic":         {"emoji": "😏", "color": _C(148), "name": "ironic",         "family": "humor", "intensity": 4, "valence": "mixed"},
    "sardonic":       {"emoji": "😒", "color": _C(100), "name": "sardonic",       "family": "humor", "intensity": 5, "valence": "mixed"},
    "dry_humor":      {"emoji": "🎩", "color": _C(100), "name": "dry_humor",      "family": "humor", "intensity": 5, "valence": "mixed"},

    # ── CULTURAL (10) ────────────────────────────────────────────────────────
    "sonder":         {"emoji": "🌍", "color": _C(179), "name": "sonder",         "family": "cultural", "intensity": 3, "valence": "neutral"},
    "hiraeth":        {"emoji": "🏔️", "color": _C(173), "name": "hiraeth",        "family": "cultural", "intensity": 4, "valence": "mixed"},
    "fernweh":        {"emoji": "🗺️", "color": _C(172), "name": "fernweh",        "family": "cultural", "intensity": 3, "valence": "mixed"},
    "sehnsucht":      {"emoji": "🌌", "color": _C(136), "name": "sehnsucht",      "family": "cultural", "intensity": 5, "valence": "mixed"},
    "schadenfreude":  {"emoji": "😏", "color": _C(136), "name": "schadenfreude",  "family": "cultural", "intensity": 3, "valence": "mixed"},
    "saudade":        {"emoji": "🌊", "color": _C(130), "name": "saudade",        "family": "cultural", "intensity": 4, "valence": "mixed"},
    "toska":          {"emoji": "🌧️", "color": _C(130), "name": "toska",          "family": "cultural", "intensity": 4, "valence": "negative"},
    "forelsket":      {"emoji": "💘", "color": _C(172), "name": "forelsket",      "family": "cultural", "intensity": 5, "valence": "positive"},
    "mono_no_aware":  {"emoji": "🍂", "color": _C(179), "name": "mono_no_aware",  "family": "cultural", "intensity": 3, "valence": "mixed"},
    "jayus":          {"emoji": "😂", "color": _C(136), "name": "jayus",          "family": "cultural", "intensity": 2, "valence": "mixed"},

    # ── FLOW (10) ────────────────────────────────────────────────────────────
    "absorbed":       {"emoji": "🎯", "color": _C(159), "name": "absorbed",       "family": "flow", "intensity": 1, "valence": "positive"},
    "immersed":       {"emoji": "🏊", "color": _C(123), "name": "immersed",       "family": "flow", "intensity": 2, "valence": "positive"},
    "focused_flow":   {"emoji": "🔭", "color": _C(87),  "name": "focused_flow",   "family": "flow", "intensity": 2, "valence": "positive"},
    "effortless":     {"emoji": "🕊️", "color": _C(87),  "name": "effortless",     "family": "flow", "intensity": 3, "valence": "positive"},
    "flowing":        {"emoji": "🌊", "color": _C(51),  "name": "flowing",        "family": "flow", "intensity": 3, "valence": "positive"},
    "fluid":          {"emoji": "💧", "color": _C(51),  "name": "fluid",          "family": "flow", "intensity": 3, "valence": "positive"},
    "hyperfocused":   {"emoji": "🔬", "color": _C(45),  "name": "hyperfocused",   "family": "flow", "intensity": 4, "valence": "positive"},
    "in_the_zone":    {"emoji": "⚡", "color": _C(45),  "name": "in_the_zone",    "family": "flow", "intensity": 4, "valence": "positive"},
    "peak":           {"emoji": "🏔️", "color": _C(39),  "name": "peak",           "family": "flow", "intensity": 5, "valence": "positive"},
    "transcending":   {"emoji": "🚀", "color": _C(39),  "name": "transcending",   "family": "flow", "intensity": 5, "valence": "positive"},

    # ── SPIRITUAL (10) ───────────────────────────────────────────────────────
    "prayerful":      {"emoji": "🙏", "color": _C(189), "name": "prayerful",      "family": "spiritual", "intensity": 1, "valence": "positive"},
    "reverent":       {"emoji": "🕯️", "color": _C(183), "name": "reverent",       "family": "spiritual", "intensity": 2, "valence": "positive"},
    "devotional":     {"emoji": "🫶", "color": _C(141), "name": "devotional",     "family": "spiritual", "intensity": 2, "valence": "positive"},
    "zenlike":        {"emoji": "☯️", "color": _C(141), "name": "zenlike",        "family": "spiritual", "intensity": 3, "valence": "positive"},
    "meditative":     {"emoji": "🧘", "color": _C(135), "name": "meditative",     "family": "spiritual", "intensity": 3, "valence": "positive"},
    "sacred":         {"emoji": "⛩️", "color": _C(99),  "name": "sacred",         "family": "spiritual", "intensity": 4, "valence": "positive"},
    "mystical":       {"emoji": "🔮", "color": _C(99),  "name": "mystical",       "family": "spiritual", "intensity": 4, "valence": "positive"},
    "awakened":       {"emoji": "🌅", "color": _C(93),  "name": "awakened",       "family": "spiritual", "intensity": 5, "valence": "positive"},
    "cosmic":         {"emoji": "🌠", "color": _C(93),  "name": "cosmic",         "family": "spiritual", "intensity": 5, "valence": "positive"},
    "numinous":       {"emoji": "✨", "color": _C(57),  "name": "numinous",       "family": "spiritual", "intensity": 5, "valence": "positive"},

    # ── SOCIAL_AWKWARD (10) ──────────────────────────────────────────────────
    "people_pleasing": {"emoji": "😅", "color": _C(223), "name": "people_pleasing", "family": "social_awkward", "intensity": 1, "valence": "negative"},
    "oversharing":    {"emoji": "🗣️", "color": _C(223), "name": "oversharing",    "family": "social_awkward", "intensity": 2, "valence": "negative"},
    "fawning":        {"emoji": "🐕", "color": _C(217), "name": "fawning",        "family": "social_awkward", "intensity": 2, "valence": "negative"},
    "awkward":        {"emoji": "😬", "color": _C(217), "name": "awkward",        "family": "social_awkward", "intensity": 3, "valence": "negative"},
    "socially_anxious": {"emoji": "😰", "color": _C(216), "name": "socially_anxious", "family": "social_awkward", "intensity": 3, "valence": "negative"},
    "clingy":         {"emoji": "🔗", "color": _C(216), "name": "clingy",         "family": "social_awkward", "intensity": 3, "valence": "negative"},
    "needy":          {"emoji": "🙏", "color": _C(210), "name": "needy",          "family": "social_awkward", "intensity": 4, "valence": "negative"},
    "cringe":         {"emoji": "😣", "color": _C(210), "name": "cringe",         "family": "social_awkward", "intensity": 4, "valence": "negative"},
    "self_sabotaging": {"emoji": "💣", "color": _C(209), "name": "self_sabotaging", "family": "social_awkward", "intensity": 5, "valence": "negative"},
    "secondhand_embarrassed": {"emoji": "😖", "color": _C(209), "name": "secondhand_embarrassed", "family": "social_awkward", "intensity": 5, "valence": "negative"},

    # ── RESILIENCE (10) ──────────────────────────────────────────────────────
    "recovering":     {"emoji": "🔄", "color": _C(157), "name": "recovering",     "family": "resilience", "intensity": 1, "valence": "positive"},
    "adapting":       {"emoji": "🦎", "color": _C(121), "name": "adapting",       "family": "resilience", "intensity": 2, "valence": "positive"},
    "learning":       {"emoji": "📚", "color": _C(85),  "name": "learning",       "family": "resilience", "intensity": 2, "valence": "positive"},
    "healing":        {"emoji": "💚", "color": _C(85),  "name": "healing",        "family": "resilience", "intensity": 3, "valence": "positive"},
    "growing":        {"emoji": "📈", "color": _C(71),  "name": "growing",        "family": "resilience", "intensity": 3, "valence": "positive"},
    "resilient":      {"emoji": "🌱", "color": _C(71),  "name": "resilient",      "family": "resilience", "intensity": 3, "valence": "positive"},
    "rebounding":     {"emoji": "🏀", "color": _C(65),  "name": "rebounding",     "family": "resilience", "intensity": 4, "valence": "positive"},
    "renewed":        {"emoji": "🌿", "color": _C(65),  "name": "renewed",        "family": "resilience", "intensity": 4, "valence": "positive"},
    "transforming":   {"emoji": "🦋", "color": _C(35),  "name": "transforming",   "family": "resilience", "intensity": 5, "valence": "positive"},
    "evolving":       {"emoji": "🔄", "color": _C(35),  "name": "evolving",       "family": "resilience", "intensity": 5, "valence": "positive"},

    # ── COMPLEXITY (10) ──────────────────────────────────────────────────────
    "nuanced":        {"emoji": "🎨", "color": _C(189), "name": "nuanced",        "family": "complexity", "intensity": 1, "valence": "neutral"},
    "layered":        {"emoji": "📚", "color": _C(183), "name": "layered",        "family": "complexity", "intensity": 2, "valence": "neutral"},
    "multifaceted":   {"emoji": "💎", "color": _C(177), "name": "multifaceted",   "family": "complexity", "intensity": 2, "valence": "neutral"},
    "in_between":     {"emoji": "🌗", "color": _C(147), "name": "in_between",     "family": "complexity", "intensity": 3, "valence": "neutral"},
    "ambiguous":      {"emoji": "🌀", "color": _C(147), "name": "ambiguous",      "family": "complexity", "intensity": 3, "valence": "neutral"},
    "oscillating":    {"emoji": "↕️", "color": _C(141), "name": "oscillating",    "family": "complexity", "intensity": 3, "valence": "neutral"},
    "contradictory":  {"emoji": "↔️", "color": _C(135), "name": "contradictory",  "family": "complexity", "intensity": 4, "valence": "mixed"},
    "intricate":      {"emoji": "🕸️", "color": _C(135), "name": "intricate",      "family": "complexity", "intensity": 4, "valence": "neutral"},
    "paradoxical":    {"emoji": "∞",  "color": _C(141), "name": "paradoxical",    "family": "complexity", "intensity": 5, "valence": "mixed"},
    "complicated":    {"emoji": "🔀", "color": _C(141), "name": "complicated",    "family": "complexity", "intensity": 5, "valence": "negative"},

    # ── NUMB (10) ────────────────────────────────────────────────────────────
    "flat":           {"emoji": "📏", "color": _C(240), "name": "flat",           "family": "numb", "intensity": 1, "valence": "neutral"},
    "going_through_motions": {"emoji": "🤖", "color": _C(238), "name": "going_through_motions", "family": "numb", "intensity": 1, "valence": "neutral"},
    "detached":       {"emoji": "🔌", "color": _C(238), "name": "detached",       "family": "numb", "intensity": 2, "valence": "neutral"},
    "disconnected":   {"emoji": "📡", "color": _C(236), "name": "disconnected",   "family": "numb", "intensity": 2, "valence": "neutral"},
    "anesthetized":   {"emoji": "💊", "color": _C(236), "name": "anesthetized",   "family": "numb", "intensity": 3, "valence": "neutral"},
    "hollow":         {"emoji": "🕳️", "color": _C(234), "name": "hollow",         "family": "numb", "intensity": 3, "valence": "negative"},
    "dissociated":    {"emoji": "👁️", "color": _C(234), "name": "dissociated",    "family": "numb", "intensity": 4, "valence": "negative"},
    "empty":          {"emoji": "⚫", "color": _C(234), "name": "empty",          "family": "numb", "intensity": 4, "valence": "negative"},
    "deadened":       {"emoji": "💀", "color": _C(232), "name": "deadened",       "family": "numb", "intensity": 5, "valence": "negative"},
    "numb":           {"emoji": "🧊", "color": _C(232), "name": "numb",           "family": "numb", "intensity": 5, "valence": "negative"},

    # ── HOPE (10) ────────────────────────────────────────────────────────────
    "positive":       {"emoji": "➕", "color": _C(229), "name": "positive",       "family": "hope", "intensity": 1, "valence": "positive"},
    "optimistic":     {"emoji": "☀️", "color": _C(229), "name": "optimistic",     "family": "hope", "intensity": 2, "valence": "positive"},
    "uplifted":       {"emoji": "🎈", "color": _C(222), "name": "uplifted",       "family": "hope", "intensity": 2, "valence": "positive"},
    "trusting":       {"emoji": "🤝", "color": _C(222), "name": "trusting",       "family": "hope", "intensity": 3, "valence": "positive"},
    "faithful":       {"emoji": "⭐", "color": _C(221), "name": "faithful",       "family": "hope", "intensity": 3, "valence": "positive"},
    "believing":      {"emoji": "🙏", "color": _C(221), "name": "believing",      "family": "hope", "intensity": 3, "valence": "positive"},
    "aspiring":       {"emoji": "🌅", "color": _C(185), "name": "aspiring",       "family": "hope", "intensity": 4, "valence": "positive"},
    "hopeful_bright": {"emoji": "🌈", "color": _C(185), "name": "hopeful_bright", "family": "hope", "intensity": 4, "valence": "positive"},
    "expectant_good": {"emoji": "🌱", "color": _C(149), "name": "expectant_good", "family": "hope", "intensity": 5, "valence": "positive"},
    "visionary_hope": {"emoji": "🔭", "color": _C(149), "name": "visionary_hope", "family": "hope", "intensity": 5, "valence": "positive"},
}

EMOTION_NAMES = list(EMOTIONS.keys())
RESET = "\033[0m"
BOLD  = "\033[1m"
DIM   = "\033[2m"



"""emotion.output — terminal logging and dashboard display."""

from .logger       import say, say_registration, running, separator, say_evolution, print_metrics_row
from .dashboard    import show_dashboard
from .conversation import display_conversation

__all__ = [
    "say", "say_registration", "running", "separator",
    "say_evolution", "print_metrics_row",
    "show_dashboard",
    "display_conversation",
]



from __future__ import annotations

from .ansi import (
    ansi_color,
    RESET, BOLD, DIM,
    GREEN, YELLOW, RED, CYAN, GRAY, ORANGE,
    health_color, complexity_color, bar,
)
from .ast_helpers import (
    cyclomatic_complexity,
    nesting_depth,
    has_recursion,
    has_heavy_computation,
)

__all__ = [
    "ansi_color", "RESET", "BOLD", "DIM",
    "GREEN", "YELLOW", "RED", "CYAN", "GRAY", "ORANGE",
    "health_color", "complexity_color", "bar",
    "cyclomatic_complexity", "nesting_depth",
    "has_recursion", "has_heavy_computation",
]



from __future__ import annotations

import ast


def cyclomatic_complexity(tree: ast.AST) -> int:
    """McCabe cyclomatic complexity: count of independent execution paths."""
    count = 1
    for node in ast.walk(tree):
        if isinstance(node, (ast.If, ast.While, ast.For, ast.ExceptHandler,
                             ast.With, ast.Assert, ast.comprehension)):
            count += 1
        elif isinstance(node, ast.BoolOp):
            count += len(node.values) - 1
    return count


def nesting_depth(tree: ast.AST) -> int:
    """Maximum nesting depth of control-flow blocks in *tree*."""
    max_depth: list[int] = [0]

    def walk_node(node: ast.AST, depth: int) -> None:
        if isinstance(node, (ast.If, ast.While, ast.For, ast.With,
                             ast.Try, ast.FunctionDef, ast.AsyncFunctionDef)):
            max_depth[0] = max(max_depth[0], depth)
            for child in ast.iter_child_nodes(node):
                walk_node(child, depth + 1)
        else:
            for child in ast.iter_child_nodes(node):
                walk_node(child, depth)

    walk_node(tree, 1)
    return max_depth[0]


def has_recursion(func_name: str, tree: ast.AST) -> bool:
    """Return True if the function body contains a call to itself."""
    for node in ast.walk(tree):
        if isinstance(node, ast.Call):
            if isinstance(node.func, ast.Name) and node.func.id == func_name:
                return True
            if isinstance(node.func, ast.Attribute) and node.func.attr == func_name:
                return True
    return False


def has_heavy_computation(tree: ast.AST) -> bool:
    """Return True if the function contains ≥2 loop/comprehension constructs."""
    count = 0
    for node in ast.walk(tree):
        if isinstance(node, (ast.For, ast.While, ast.ListComp,
                             ast.GeneratorExp, ast.SetComp, ast.DictComp)):
            count += 1
    return count >= 2



"""quickstart.py — emotion library usage demo.

Run from the repo root:
    python examples/quickstart.py
"""
import sys
sys.path.insert(0, ".")

from checkemotion import emotion, show_dashboard


@emotion
def calculate_total(numbers: list) -> float:
    """Sum positive numbers in a list."""
    return sum(n for n in numbers if n > 0)


@emotion
def validate_user(user_id: int, role: str) -> bool:
    """Check that user_id is positive and role is non-empty."""
    try:
        assert user_id > 0, "user_id must be positive"
        assert role.strip(), "role must not be empty"
        return True
    except AssertionError:
        return False


@emotion
def get_config(key: str) -> str:
    """Retrieve a config value by key."""
    defaults = {"host": "localhost", "port": "8080", "debug": "false"}
    return defaults.get(key, "")


@emotion
def delete_expired_sessions(sessions: list) -> list:
    """Remove all expired sessions from the list."""
    return [s for s in sessions if not s.get("expired", False)]


@emotion
def generate_report(data: dict) -> dict:
    """Build and return a structured report dict from raw data."""
    report = {}
    for key, value in data.items():
        if isinstance(value, (int, float)):
            report[key] = {"value": value, "type": "numeric"}
        else:
            report[key] = {"value": str(value), "type": "text"}
    return report


if __name__ == "__main__":
    print("\n" + "=" * 60)
    print("  Running example calls...")
    print("=" * 60 + "\n")

    calculate_total([1, -2, 3, 4, -5])
    calculate_total([10, 20, 30])

    validate_user(42, "admin")
    validate_user(0, "")          # triggers exception path

    get_config("host")
    get_config("port")

    delete_expired_sessions([
        {"id": 1, "expired": False},
        {"id": 2, "expired": True},
    ])

    generate_report({"users": 150, "revenue": 9800.50, "plan": "pro"})

    print()
    show_dashboard()



from __future__ import annotations

import os
from typing import Literal, Optional

from ..core import registry
from ..schemas.emotions import EMOTIONS, RESET, BOLD, DIM
from ..utils.ansi import (
    health_color, complexity_color, bar,
    CYAN, GRAY, ORANGE, RED, YELLOW, GREEN,
)
from ..analysis.metrics import FunctionReport, all_reports


def render_report(report: FunctionReport, max_call_count: int, terminal_width: int) -> None:
    emotion_data          = EMOTIONS.get(report.emotion, EMOTIONS["content"])
    emoji                 = emotion_data.get("emoji", "❓")
    emotion_color         = emotion_data.get("color", "")
    health_value          = report.health_score()
    health_display_color  = health_color(health_value)
    complexity_display_color = complexity_color(report.cyclomatic_complexity)
    central_marker        = f"{YELLOW}★{RESET} " if report.is_central else "  "

    # ── Header ─────────────────────────────────────────────────────────────────
    print(
        f"{central_marker}{emotion_color}{BOLD}{emoji}  {report.name}{RESET}"
        f"  {DIM}[{report.emotion}]{RESET}"
        f"  {health_display_color}{report.star_rating()}  health={health_value:.0f}/100{RESET}"
    )

    # ── Description ────────────────────────────────────────────────────────────
    if report.description:
        max_description_length = terminal_width - 6
        display_description = report.description[:max_description_length]
        if len(report.description) > max_description_length:
            display_description = display_description[:-1] + "…"
        print(f"  {GRAY}↳ {display_description}{RESET}")

    # ── Static analysis ────────────────────────────────────────────────────────
    behavior_flags = []
    if report.has_recursion:  behavior_flags.append("recursive")
    if report.has_async:      behavior_flags.append("async")
    if report.has_try_except: behavior_flags.append("try/except")
    if report.has_yield:      behavior_flags.append("generator")
    behavior_flags_text = f"  {GRAY}[{', '.join(behavior_flags)}]{RESET}" if behavior_flags else ""

    print(
        f"  {GRAY}static  {RESET}"
        f"lines={BOLD}{report.line_count}{RESET}"
        f"  params={BOLD}{report.param_count}{RESET}"
        f"  branches={BOLD}{report.branch_count}{RESET}"
        f"  nesting={BOLD}{report.nesting_depth:.0f}{RESET}"
        f"  cc={complexity_display_color}{BOLD}{report.cyclomatic_complexity:.0f}{RESET} "
        f"{GRAY}({report.complexity_tier()}){RESET}"
        f"{behavior_flags_text}"
    )

    # ── Runtime metrics ────────────────────────────────────────────────────────
    call_count_bar        = bar(report.call_count, max_call_count)
    exception_percentage  = f"{report.exception_rate * 100:.0f}%"
    exception_display_color = (
        RED    if report.exception_rate > 0.1 else
        YELLOW if report.exception_rate > 0   else
        GREEN
    )

    print(
        f"  {GRAY}runtime {RESET}"
        f"calls={BOLD}{report.call_count}{RESET} {call_count_bar}"
        f"  errors={exception_display_color}{BOLD}{report.exception_count}{RESET}"
        f"{GRAY}({exception_percentage}){RESET}"
        f"  avg={BOLD}{report.average_time_ms:.1f}ms{RESET}"
        f"  total={GRAY}{report.total_time_ms:.0f}ms{RESET}"
    )

    # ── Profiling data (shown only if calls were profiled) ─────────────────────
    if report.average_wall_time_ms > 0 or report.peak_memory_kb > 0:
        memory_display_color = (
            RED    if report.peak_memory_kb > 10240 else
            YELLOW if report.peak_memory_kb > 1024  else
            GREEN
        )
        print(
            f"  {GRAY}profile {RESET}"
            f"wall={BOLD}{report.average_wall_time_ms:.1f}ms{RESET}"
            f"  cpu={BOLD}{report.average_cpu_time_ms:.2f}ms{RESET}"
            f"  mem_avg=+{BOLD}{report.average_memory_delta_kb:.1f}KB{RESET}"
            f"  mem_peak={memory_display_color}{BOLD}{report.peak_memory_kb:.1f}KB{RESET}"
        )

    # ── Call graph ─────────────────────────────────────────────────────────────
    if report.callers or report.callees:
        callers_display = ", ".join(report.callers[:5]) if report.callers else "—"
        callees_display = ", ".join(report.callees[:5]) if report.callees else "—"
        depth_display   = f"  depth={report.max_call_depth}" if report.max_call_depth > 0 else ""
        print(
            f"  {GRAY}graph   {RESET}"
            f"callers=[{CYAN}{callers_display}{RESET}]"
            f"  calls=[{CYAN}{callees_display}{RESET}]"
            f"{depth_display}"
        )

    # ── Social relationships ────────────────────────────────────────────────────
    if report.relationships:
        relationship_items = list(report.relationships.items())[:5]
        relationship_parts = [
            f"{ORANGE}{relationship_type}{RESET}→{other_name}"
            for other_name, relationship_type in relationship_items
        ]
        print(f"  {GRAY}social  {RESET}{', '.join(relationship_parts)}")

    print()


SortKey = Literal["call_count", "health", "complexity", "name"]


def show_dashboard(
    top_n:   Optional[int] = None,
    sort_by: SortKey        = "call_count",
) -> None:
    """Print a color-coded terminal dashboard for all registered functions."""
    reports = all_reports(sort_by=sort_by)
    if not reports:
        print(f"{DIM}No emotional functions registered yet.{RESET}")
        return

    if top_n:
        reports = reports[:top_n]

    try:
        terminal_width = min(os.get_terminal_size().columns, 120)
    except OSError:
        terminal_width = 120
    max_call_count = max((report.call_count for report in reports), default=1) or 1

    print(f"\n{BOLD}{'═' * terminal_width}{RESET}")
    print(f"{BOLD}  EMOTION AI  ·  FUNCTION DASHBOARD  ·  {len(reports)} function(s){RESET}")
    print(f"{BOLD}{'═' * terminal_width}{RESET}\n")

    for report in reports:
        render_report(report, max_call_count, terminal_width)

    # ── Summary footer ─────────────────────────────────────────────────────────
    total_call_count      = sum(report.call_count      for report in reports)
    total_exception_count = sum(report.exception_count for report in reports)
    overall_exception_rate = total_exception_count / total_call_count if total_call_count else 0.0
    average_health_score  = sum(report.health_score() for report in reports) / len(reports)

    print(f"{BOLD}{'─' * terminal_width}{RESET}")
    print(
        f"  {GRAY}TOTALS{RESET}"
        f"  functions={BOLD}{len(reports)}{RESET}"
        f"  calls={BOLD}{total_call_count}{RESET}"
        f"  errors={BOLD}{total_exception_count}{RESET}"
        f"{GRAY}({overall_exception_rate * 100:.1f}%){RESET}"
        f"  avg_health={health_color(average_health_score)}{BOLD}{average_health_score:.0f}/100{RESET}"
    )
    print(f"{BOLD}{'═' * terminal_width}{RESET}\n")



from __future__ import annotations

ansi_color = lambda n: f"\033[38;5;{n}m"  # noqa: E731

RESET = "\033[0m"
BOLD  = "\033[1m"
DIM   = "\033[2m"

GREEN  = ansi_color(82)
YELLOW = ansi_color(220)
RED    = ansi_color(196)
CYAN   = ansi_color(87)
GRAY   = ansi_color(245)
WHITE  = ansi_color(255)
ORANGE = ansi_color(214)


def health_color(score: float) -> str:
    if score >= 80: return GREEN
    if score >= 50: return YELLOW
    return RED


def complexity_color(cc: float) -> str:
    if cc <= 2:  return GREEN
    if cc <= 5:  return YELLOW
    if cc <= 10: return ORANGE
    return RED


def bar(value: float, max_val: float, width: int = 12) -> str:
    if max_val <= 0:
        return f"{GRAY}{'░' * width}{RESET}"
    filled = min(int(round(value / max_val * width)), width)
    return f"{CYAN}{'█' * filled}{GRAY}{'░' * (width - filled)}{RESET}"



from __future__ import annotations

import ast
import os
import threading
from collections import defaultdict
from pathlib import Path
from typing import Dict, List, Set

call_graph_lock = threading.Lock()
CALL_GRAPH:    Dict[str, Set[str]] = defaultdict(set)
REVERSE_GRAPH: Dict[str, Set[str]] = defaultdict(set)
SCANNED_DIRS:  Set[str] = set()


def callees_in_func(func_node: ast.FunctionDef | ast.AsyncFunctionDef) -> Set[str]:
    found: Set[str] = set()
    for node in ast.walk(func_node):
        if isinstance(node, ast.Call):
            if isinstance(node.func, ast.Name):
                found.add(node.func.id)
            elif isinstance(node.func, ast.Attribute):
                found.add(node.func.attr)
    return found


def scan_directory(directory: str | Path) -> None:
    directory = str(Path(directory).resolve())
    with call_graph_lock:
        if directory in SCANNED_DIRS:
            return
        SCANNED_DIRS.add(directory)

    for root, _, files in os.walk(directory):
        for fname in files:
            if not fname.endswith(".py"):
                continue
            fpath = os.path.join(root, fname)
            try:
                source = Path(fpath).read_text(encoding="utf-8", errors="ignore")
                tree = ast.parse(source, filename=fpath)
            except (SyntaxError, OSError, ValueError):
                continue

            for node in ast.walk(tree):
                if isinstance(node, (ast.FunctionDef, ast.AsyncFunctionDef)):
                    caller = node.name
                    for callee in callees_in_func(node):
                        if callee == caller:
                            continue
                        with call_graph_lock:
                            CALL_GRAPH[caller].add(callee)
                            REVERSE_GRAPH[callee].add(caller)


def callers_of(name: str) -> List[str]:
    with call_graph_lock:
        return sorted(REVERSE_GRAPH.get(name, set()))


def callees_of(name: str) -> List[str]:
    with call_graph_lock:
        return sorted(CALL_GRAPH.get(name, set()))


def centrality(name: str) -> int:
    with call_graph_lock:
        return len(REVERSE_GRAPH.get(name, set()))


def is_central(name: str, threshold: int = 3) -> bool:
    return centrality(name) >= threshold


def call_depth(name: str, max_depth: int = 6) -> int:
    visited: Set[str] = set()
    queue = [(name, 0)]
    max_seen = 0
    while queue:
        cur, depth = queue.pop(0)
        if cur in visited or depth > max_depth:
            continue
        visited.add(cur)
        max_seen = max(max_seen, depth)
        with call_graph_lock:
            children = list(CALL_GRAPH.get(cur, set()))
        for child in children:
            queue.append((child, depth + 1))
    return max_seen



from __future__ import annotations

import ast
import inspect
import json
import textwrap
import urllib.error
import urllib.request
from typing import Optional

OLLAMA_GENERATE = "http://localhost:11434/api/generate"
OLLAMA_TAGS     = "http://localhost:11434/api/tags"

PREFERRED_MODELS = [
    "phi3.5:mini", "phi3:mini", "phi3:3.8b",
    "llama3.2:1b", "llama3.2:3b", "tinyllama",
    "mistral:7b-instruct-q2_K",
]

ollama_model: Optional[str] = None
ollama_checked: bool = False


def probe_ollama() -> Optional[str]:
    try:
        resp = urllib.request.urlopen(OLLAMA_TAGS, timeout=1)
        data = json.loads(resp.read())
        available: dict[str, str] = {
            m["name"].split(":")[0]: m["name"]
            for m in data.get("models", [])
        }
        for pref in PREFERRED_MODELS:
            base = pref.split(":")[0]
            if base in available:
                return available[base]
        first = data.get("models", [])
        if first:
            return first[0]["name"]
    except Exception:
        pass
    return None


def ollama_describe(source: str, func_name: str, model: str) -> Optional[str]:
    prompt = (
        "In exactly one concise plain-English sentence (no code, no bullet points), "
        "describe what this Python function does from a developer's perspective:\n\n"
        f"{source}\n\nOne sentence:"
    )
    payload = json.dumps({
        "model": model,
        "prompt": prompt,
        "stream": False,
        "options": {"num_predict": 100, "temperature": 0.1},
    }).encode()
    try:
        req = urllib.request.Request(
            OLLAMA_GENERATE, data=payload,
            headers={"Content-Type": "application/json"}, method="POST",
        )
        resp = urllib.request.urlopen(req, timeout=8)
        text = json.loads(resp.read()).get("response", "").strip()
        if text:
            return text.split(".")[0].strip() + "."
    except Exception:
        pass
    return None


def infer_verb(name: str) -> str:
    n = name.lower()
    mapping = [
        (["get", "fetch", "load", "read", "retrieve", "find", "lookup"],    "Retrieves"),
        (["set", "put", "write", "save", "store", "persist", "push"],        "Stores"),
        (["create", "make", "build", "generate", "new", "produce", "forge"], "Creates"),
        (["delete", "remove", "drop", "clear", "destroy", "purge"],          "Removes"),
        (["update", "edit", "modify", "change", "patch", "mutate"],          "Updates"),
        (["check", "validate", "verify", "assert", "ensure", "is_", "has_"], "Validates"),
        (["parse", "decode", "extract", "analyze", "scan"],                  "Analyzes"),
        (["send", "emit", "publish", "notify", "dispatch", "broadcast"],     "Sends"),
        (["run", "execute", "process", "handle", "perform", "apply"],        "Executes"),
        (["init", "setup", "configure", "prepare", "bootstrap"],             "Initializes"),
        (["format", "render", "display", "print", "show", "log", "output"],  "Displays"),
        (["convert", "transform", "map", "translate", "encode"],             "Converts"),
        (["sort", "order", "rank", "prioritize"],                            "Sorts"),
        (["filter", "search", "query"],                                      "Searches"),
        (["calc", "compute", "sum", "count", "measure", "aggregate"],        "Computes"),
        (["listen", "watch", "subscribe", "poll", "wait"],                   "Watches"),
        (["connect", "join", "register", "bind", "attach"],                  "Connects"),
        (["close", "disconnect", "release", "shutdown", "stop"],             "Closes"),
    ]
    for keywords, verb in mapping:
        if any(k in n for k in keywords):
            return verb
    return "Processes"


def ast_describe(func) -> str:
    try:
        src = textwrap.dedent(inspect.getsource(func))
        tree = ast.parse(src)
    except (OSError, TypeError, SyntaxError):
        return "Processes data."

    fnodes = [n for n in ast.walk(tree)
              if isinstance(n, (ast.FunctionDef, ast.AsyncFunctionDef))]
    if not fnodes:
        return "Processes data."
    fn = fnodes[0]

    is_async      = isinstance(fn, ast.AsyncFunctionDef)
    params        = [p.arg for p in fn.args.args if p.arg not in ("self", "cls")]
    calls: list   = []
    returns_val   = False
    raises        = False
    yields        = False
    loops         = False
    conditions    = 0

    for node in ast.walk(fn):
        if isinstance(node, ast.Call):
            if isinstance(node.func, ast.Name):
                calls.append(node.func.id)
            elif isinstance(node.func, ast.Attribute):
                calls.append(node.func.attr)
        elif isinstance(node, ast.Return) and node.value is not None:
            returns_val = True
        elif isinstance(node, ast.Raise):
            raises = True
        elif isinstance(node, (ast.Yield, ast.YieldFrom)):
            yields = True
        elif isinstance(node, (ast.For, ast.While)):
            loops = True
        elif isinstance(node, ast.If):
            conditions += 1

    verb = infer_verb(func.__name__)
    parts: list[str] = []

    if is_async:
        parts.append("Asynchronously")

    parts.append(verb)

    if params:
        if len(params) == 1:
            parts.append(f"the {params[0].replace('_', ' ')}")
        else:
            listed = ", ".join(p.replace("_", " ") for p in params[:3])
            parts.append(f"inputs ({listed})")

    if loops:
        parts.append("by iterating over items")

    if conditions > 2:
        parts.append("across multiple conditions")

    key_calls = [c for c in dict.fromkeys(calls) if not c.startswith("_")][:3]
    if key_calls:
        parts.append(f"using {', '.join(key_calls)}")

    if yields:
        parts.append("and yields results lazily")
    elif returns_val:
        parts.append("and returns the result")

    if raises:
        parts.append(", raising on invalid input")

    return " ".join(parts).rstrip(",") + "."


def describe(func) -> str:
    global ollama_model, ollama_checked

    if not ollama_checked:
        ollama_model = probe_ollama()
        ollama_checked = True

    if ollama_model:
        try:
            src = textwrap.dedent(inspect.getsource(func))
        except (OSError, TypeError):
            src = ""
        if src:
            result = ollama_describe(src, func.__name__, ollama_model)
            if result:
                return result

    return ast_describe(func)



from __future__ import annotations

import os
import random
from typing import Optional

from ..utils.ansi import RESET, BOLD, DIM, CYAN
from ..utils.dialogue_data import EMOTION_VOICES, RELATIONSHIP_EXCHANGES, STAT_BANTER
from ..utils.emotion_lookup import ALL_EMOTIONS, RELATIONSHIP_COLORS

try:
    from ..core.registry import get_data as registry_get_data
except Exception:
    registry_get_data = None


def get_emotion_family(emotion: str) -> str:
    return ALL_EMOTIONS.get(emotion, {}).get("family", emotion)


def get_emotion_color(emotion: str) -> str:
    return ALL_EMOTIONS.get(emotion, {}).get("color", "")


def get_emotion_emoji(emotion: str) -> str:
    return ALL_EMOTIONS.get(emotion, {}).get("emoji", "🤖")


def pick_voice(emotion: str) -> str:
    family = get_emotion_family(emotion)
    phrases = EMOTION_VOICES.get(family, EMOTION_VOICES.get(emotion, [f"Running as {emotion}."]))
    return random.choice(phrases)


def generate_dialogue(
    func_a: str, stats_a: dict,
    func_b: str, stats_b: dict,
    emotion_a: str, emotion_b: str,
    relationship: Optional[str],
) -> tuple[str, str]:
    sa = {
        "calls":  stats_a.get("call_count", 0),
        "errors": stats_a.get("exception_count", 0),
        "ms":     stats_a.get("average_time_ms", 0.0),
    }
    sb = {
        "calls":  stats_b.get("call_count", 0),
        "errors": stats_b.get("exception_count", 0),
        "ms":     stats_b.get("average_time_ms", 0.0),
    }

    def format_banter_line(tmpl: str) -> str:
        pct_a = int(sa["errors"] / sa["calls"] * 100) if sa["calls"] else 0
        ok_a  = sa["calls"] - sa["errors"]
        return tmpl.format(
            a=func_a, b=func_b,
            calls_a=sa["calls"], errors_a=sa["errors"], ms_a=f"{sa['ms']:.1f}",
            calls_b=sb["calls"], errors_b=sb["errors"], ms_b=f"{sb['ms']:.1f}",
            pct_a=pct_a, ok_a=ok_a,
        )

    candidates = []
    for condition, templates in STAT_BANTER:
        try:
            if condition(sa, sb):
                candidates.extend(templates)
        except Exception:
            pass
    if candidates:
        raw_a, raw_b = random.choice(candidates)
        try:
            return format_banter_line(raw_a), format_banter_line(raw_b)
        except Exception:
            pass

    if relationship and relationship in RELATIONSHIP_EXCHANGES:
        init_phrases, resp_phrases = RELATIONSHIP_EXCHANGES[relationship]
        return random.choice(init_phrases), random.choice(resp_phrases)

    return pick_voice(emotion_a), pick_voice(emotion_b)


def display_conversation(
    func_a: str,
    emotion_a: str,
    func_b: str,
    emotion_b: str,
    relationship: str | None,
) -> None:
    try:
        width = min(os.get_terminal_size().columns, 100)
    except OSError:
        width = 90

    stats_a: dict = {}
    stats_b: dict = {}
    if registry_get_data is not None:
        try:
            stats_a = registry_get_data(func_a) or {}
            stats_b = registry_get_data(func_b) or {}
        except Exception:
            pass

    color_a = get_emotion_color(emotion_a)
    color_b = get_emotion_color(emotion_b)
    emoji_a = get_emotion_emoji(emotion_a)
    emoji_b = get_emotion_emoji(emotion_b)

    inner_w   = width - 4
    title     = " ⚡ FUNCTION DIALOGUE "
    title_pad = title.center(inner_w, "─")

    print(f"\n{DIM}╔{'═' * inner_w}╗{RESET}")
    print(f"{DIM}║{RESET}{BOLD}{title_pad}{RESET}{DIM}║{RESET}")
    print(f"{DIM}╠{'═' * inner_w}╣{RESET}")

    if relationship:
        rel_color = RELATIONSHIP_COLORS.get(relationship, CYAN)
        rel_line = (
            f"  {rel_color}{BOLD}{relationship.upper()}{RESET}"
            f"  ·  {color_a}{BOLD}{emoji_a} {func_a}{RESET} [{emotion_a}]"
            f"  ↔  {color_b}{BOLD}{emoji_b} {func_b}{RESET} [{emotion_b}]"
        )
        print(f"{DIM}║{RESET} {rel_line}")
        print(f"{DIM}╠{'─' * inner_w}╣{RESET}")

    label_a = f"{color_a}{BOLD}{emoji_a} {func_a}{RESET}"
    label_b = f"{color_b}{BOLD}{emoji_b} {func_b}{RESET}"

    line_a, line_b = generate_dialogue(
        func_a, stats_a, func_b, stats_b, emotion_a, emotion_b, relationship
    )
    print(f"{DIM}║{RESET}  {label_a}  {DIM}says:{RESET}  {line_a}")
    print(f"{DIM}║{RESET}  {label_b}  {DIM}says:{RESET}  {line_b}")

    print(f"{DIM}╚{'═' * inner_w}╝{RESET}\n")



"""emotion.analysis — call graph, metrics, and function narration."""

from .call_graph import (
    scan_directory, callers_of, callees_of,
    centrality, is_central, call_depth,
)
from .metrics import build_report, all_reports, FunctionReport
from .narrator import describe

__all__ = [
    "scan_directory", "callers_of", "callees_of",
    "centrality", "is_central", "call_depth",
    "build_report", "all_reports", "FunctionReport",
    "describe",
]



"""analyzer.py — static AST feature extraction and emotion classification."""
from __future__ import annotations

import ast
import inspect
import textwrap

from ..schemas.emotions import EMOTIONS
from ..utils.ast_helpers import (
    cyclomatic_complexity as compute_cyclomatic,
    nesting_depth         as compute_nesting,
    has_recursion         as detect_recursion,
    has_heavy_computation as detect_heavy,
)


# ── Keyword flags (indices 10-14 in feature vector) ──────────────────────────
NAME_KEYWORD_FLAGS = {
    0: ["delete", "remove", "destroy", "drop", "kill", "purge", "wipe", "terminate", "abort"],
    1: ["get", "fetch", "read", "load", "receive", "pull", "retrieve", "query", "find"],
    2: ["create", "build", "make", "generate", "construct", "produce", "craft", "compose", "forge"],
    3: ["validate", "check", "verify", "assert", "ensure", "confirm", "inspect", "audit", "test"],
    4: ["log", "print", "display", "show", "render", "format", "output", "report", "emit", "debug"],
}


# ── Feature extraction ────────────────────────────────────────────────────────

def extract_features(func) -> list[float]:
    """Extract a 15-float feature vector from *func*.

    Indices
    -------
    0  branch_count          – if/for/while/except nodes
    1  line_count            – non-blank, non-decorator lines
    2  param_count           – all parameter kinds
    3  has_recursion         – 1.0 if self-call detected
    4  has_async             – 1.0 if async def
    5  has_try_except        – 1.0 if try block present
    6  has_yield             – 1.0 if yield/yield from present
    7  cyclomatic_complexity – McCabe score
    8  nesting_depth         – maximum control-flow depth
    9  return_count          – number of return statements
    10-14 name_flags         – keyword hint bitmask (5 emotion categories)

    Returns ``[0.0] * 15`` for builtins, lambdas, or unparseable sources.
    """
    try:
        src  = textwrap.dedent(inspect.getsource(func))
        tree = ast.parse(src)
    except (OSError, TypeError, SyntaxError):
        return [0.0] * 15

    func_node = None
    for node in ast.walk(tree):
        if isinstance(node, (ast.FunctionDef, ast.AsyncFunctionDef)):
            if node.name == func.__name__:
                func_node = node
                break
    if func_node is None:
        for node in ast.walk(tree):
            if isinstance(node, (ast.FunctionDef, ast.AsyncFunctionDef)):
                func_node = node
                break
    if func_node is None:
        return [0.0] * 15

    branch_count = sum(
        1 for n in ast.walk(func_node)
        if isinstance(n, (ast.If, ast.For, ast.While, ast.ExceptHandler))
    )
    try:
        lines      = inspect.getsource(func).splitlines()
        line_count = len([l for l in lines if l.strip() and not l.strip().startswith("@")])
    except OSError:
        line_count = func_node.end_lineno - func_node.lineno + 1

    args        = func_node.args
    param_count = (len(args.args) + len(args.posonlyargs) +
                   len(args.kwonlyargs) + (1 if args.vararg else 0) +
                   (1 if args.kwarg else 0))

    has_recursion  = float(detect_recursion(func.__name__, func_node))
    has_async      = float(isinstance(func_node, ast.AsyncFunctionDef))
    has_try_except = float(any(isinstance(n, ast.Try) for n in ast.walk(func_node)))
    has_yield      = float(any(isinstance(n, (ast.Yield, ast.YieldFrom)) for n in ast.walk(func_node)))
    cyclomatic_score = compute_cyclomatic(func_node)
    nesting_score    = compute_nesting(func_node)
    return_count     = sum(1 for n in ast.walk(func_node) if isinstance(n, ast.Return))

    name_lower = func.__name__.lower()
    flags = [float(any(kw in name_lower for kw in NAME_KEYWORD_FLAGS[i])) for i in range(5)]

    return [
        float(branch_count), float(line_count), float(param_count),
        has_recursion, has_async, has_try_except, has_yield,
        float(cyclomatic_score), float(nesting_score), float(return_count),
    ] + flags


# ── Family scorer ─────────────────────────────────────────────────────────────

def score_families(features: list[float], name: str,
                   is_method_flag: bool) -> dict[str, float]:
    """Return a dict of family → score based on the 15-feature vector.

    Feature index reference
    -----------------------
    0  branch_count          7  cyclomatic_complexity
    1  line_count            8  nesting_depth
    2  param_count           9  return_count
    3  has_recursion         10 keyword_flag_dramatic
    4  has_async             11 keyword_flag_humble
    5  has_try_except        12 keyword_flag_excited
    6  has_yield             13 keyword_flag_nervous
                             14 keyword_flag_sarcastic
    """
    name_lower = name.lower()

    complexity = features[7] + features[8] + features[0] * 0.5
    simplicity = max(0.0, 15.0 - complexity)
    line_count = features[1]

    def has_keyword(*words: str) -> bool:
        return any(word in name_lower for word in words)

    family_scores: dict[str, float] = {}

    family_scores["joy"]            = features[12]*4 + simplicity*0.25 + has_keyword("init","setup","new","welcome")*2 - features[10]*2
    family_scores["serenity"]       = features[6]*4 + simplicity*0.5 - features[5]*0.5 - features[10]*2 + (features[0] < 3)*1.5
    family_scores["gratitude"]      = features[11]*4 + (line_count < 10)*2 + (features[2] <= 2)*1 + features[9]*0.4 + has_keyword("ack","thank","receive")*2
    family_scores["love"]           = is_method_flag*2 + features[2]*0.4 + has_keyword("send","notify","share","push","dispatch")*3
    family_scores["longing"]        = (5 < line_count < 30)*1.5 + (not any(features[10:15]))*1 + has_keyword("old","prev","last","hist","cache","legacy")*3
    family_scores["sadness"]        = (line_count < 8 and complexity < 3)*2 + has_keyword("fail","error","miss","none","null","empty")*2
    family_scores["disappointment"] = has_keyword("fail","invalid","none","missing","not_found","404")*3 + features[9]*0.5 + (features[0] > 0 and complexity < 5)*1
    family_scores["loneliness"]     = (line_count <= 5)*3 + (features[2] == 0)*2 + (features[9] == 0)*1 + (not is_method_flag)*0.5
    family_scores["fear"]           = features[5]*3 + features[8]*1.5 + has_keyword("danger","risk","unsafe","critical","panic","emergency")*3
    family_scores["anxiety"]        = features[13]*4 + features[5]*2 + features[7]*0.3 + (features[8] >= 2)*1 + has_keyword("retry","attempt","try","safe")*1
    family_scores["vulnerability"]  = (complexity < 5)*2 + features[5]*2 + (features[2] <= 2)*1.5 - features[12]*1
    family_scores["anger"]          = features[10]*6 + features[7]*0.4 + features[0]*0.3 + has_keyword("force","hard","override","abort","kill")*2
    family_scores["irritation"]     = features[0]*1.5 + (features[7] > 5)*2 + (features[8] > 2)*1 - features[12]*1 + has_keyword("handle","catch","wrap")*1
    family_scores["resentment"]     = has_keyword("reject","deny","block","revoke","ban","forbid")*3 + features[7]*0.4 + features[0]*0.4
    family_scores["jealousy"]       = (features[2] < 2 and features[7] > 4)*2 + has_keyword("compare","rank","score","vs","versus","beat")*3
    family_scores["disgust"]        = features[13]*2 + features[0]*0.5 + has_keyword("clean","filter","sanitize","strip","escape","encode")*3
    family_scores["surprise"]       = features[4]*4 + has_keyword("unexpected","sudden","random","event","hook","trigger","callback")*2 + (features[3] and features[4])*2
    family_scores["wonder"]         = features[3]*2 + has_keyword("explore","discover","search","find","scan","crawl","traverse")*3 + features[7]*0.2
    family_scores["anticipation"]   = features[12]*2 + features[4]*3 + has_keyword("wait","poll","watch","listen","subscribe","queue","schedule")*3
    family_scores["confidence"]     = features[2]*0.4 + features[12]*1.5 + features[9]*0.4 + (features[7] > 3 and features[8] < 4)*2 + has_keyword("run","execute","apply","do")*1
    family_scores["ambition"]       = line_count*0.05 + features[12]*2 + features[9]*0.5 + (line_count > 30)*2 + has_keyword("pipeline","workflow","orchestrate","manage")*2
    family_scores["accomplishment"] = features[9]*2 + has_keyword("complete","finish","done","result","output","return","resolve")*3 + (features[7] > 2)*1
    family_scores["humility"]       = features[11]*5 + (line_count < 15)*2 + (features[2] <= 3)*1 - features[10]*2 + has_keyword("helper","util","simple","basic")*2
    family_scores["intellectual"]   = features[7]*0.8 + features[13]*2 + has_keyword("analyze","compute","calc","process","parse","eval","solve")*3
    family_scores["creative"]       = features[12]*3 + (line_count > 20)*1.5 + has_keyword("generate","design","craft","compose","render","template")*3 + features[6]*1
    family_scores["confusion"]      = features[7]*0.5 + features[8]*1.5 + features[0]*0.8 - features[12]*1 - features[11]*1 + (complexity > 12)*2
    family_scores["indecision"]     = features[9]*1.5 + features[0]*0.8 + (not any(features[10:15]) and features[0] > 3)*2 + has_keyword("maybe","optional","fallback","default")*2
    family_scores["boredom"]        = features[14]*4 + (line_count < 5)*3 + (complexity < 2)*2 - features[12]*2 + has_keyword("noop","pass","stub","placeholder")*3
    family_scores["exhaustion"]     = line_count*0.06 + (line_count > 40)*2 + features[0]*0.4 - features[12]*1 + has_keyword("long","heavy","batch","bulk","all")*1
    family_scores["overwhelm"]      = features[7]*0.7 + features[8]*2 + features[0]*0.6 + (complexity > 15)*3 + (features[5] and features[0] > 5)*2
    family_scores["empowerment"]    = features[2]*0.8 + features[12]*2 + has_keyword("enable","allow","grant","unlock","authorize","activate")*3
    family_scores["belonging"]      = is_method_flag*4 + has_keyword("join","add","member","group","register","connect","bind")*3
    family_scores["betrayal"]       = features[10]*3 + has_keyword("override","replace","revoke","bypass","hijack","intercept","mock")*3 + features[5]*1.5
    family_scores["moral"]          = features[13]*3 + has_keyword("enforce","require","must","policy","rule","constraint","guard")*3
    family_scores["shame_deep"]     = has_keyword("fail","error","wrong","bad","invalid","broken","corrupt","oops")*3 + (complexity < 3)*2 - features[12]*2
    family_scores["existential"]    = features[3]*4 + (not any(features[10:15]) and features[7] > 5)*2 + has_keyword("self","meta","reflect","recurse","core","root")*2
    family_scores["power"]          = features[2]*1 + features[7]*0.5 + has_keyword("control","manage","admin","master","root","super","command")*3
    family_scores["humor"]          = features[14]*5 + has_keyword("debug","mock","fake","dummy","test","stub","joke","fun","random")*2
    family_scores["cultural"]       = has_keyword("transform","convert","translate","migrate","adapt","localize","encode","decode")*2 + (features[3] and features[6])*2
    family_scores["flow"]           = features[6]*5 + has_keyword("stream","pipe","chain","iter","yield","generate","next","flow")*3 + (3 < features[7] < 8)*1
    family_scores["spiritual"]      = features[3]*2 + features[6]*2 + has_keyword("soul","spirit","sacred","ritual","prayer","divine","eternal")*4
    family_scores["social_awkward"] = (is_method_flag and features[2] > 4)*2 + (features[5] and is_method_flag)*2 + has_keyword("callback","handler","middleware","intercept")*1
    family_scores["resilience"]     = features[5]*3 + features[3]*2 + has_keyword("retry","recover","restore","heal","backup","fallback","repair","fix")*3
    family_scores["complexity"]     = (features[7] > 10)*3 + (features[8] > 5)*2 + features[7]*0.3 + features[8]*0.5 - features[11]*1
    family_scores["numb"]           = (complexity < 1 and line_count <= 3 and not any(features[10:15]))*5 + (complexity < 2 and not any(features[10:15]))*2
    family_scores["hope"]           = features[12]*2 + has_keyword("init","start","begin","new","setup","boot","launch","open","create")*3 + (simplicity > 8)*1

    return family_scores


# ── Emotion classifier ────────────────────────────────────────────────────────

FAMILY_EMOTIONS: dict[str, list] = {}


def build_family_cache() -> None:
    for emotion_name, emotion_data in EMOTIONS.items():
        family_name = emotion_data["family"]
        if family_name not in FAMILY_EMOTIONS:
            FAMILY_EMOTIONS[family_name] = []
        FAMILY_EMOTIONS[family_name].append((emotion_name, emotion_data["intensity"]))
    for family_name in FAMILY_EMOTIONS:
        FAMILY_EMOTIONS[family_name].sort(key=lambda entry: entry[1])


def classify_emotion(features: list[float], func_name: str = "",
                     is_method: bool = False) -> str:
    if not FAMILY_EMOTIONS:
        build_family_cache()

    scores      = score_families(features, func_name, is_method)
    best_family = max(scores, key=scores.get)

    complexity  = features[7] + features[8] + features[0] * 0.5
    intensity   = min(5, max(1, int(complexity / 4) + 1))

    candidates  = FAMILY_EMOTIONS.get(best_family, [])
    if not candidates:
        return "content"

    return min(candidates, key=lambda x: abs(x[1] - intensity))[0]


# ── Utilities ─────────────────────────────────────────────────────────────────

def get_doc_lines(func) -> int:
    doc = inspect.getdoc(func)
    return len(doc.splitlines()) if doc else 0


def is_method(func) -> bool:
    """Heuristic: returns True if the first parameter is ``self`` or ``cls``."""
    try:
        params = list(inspect.signature(func).parameters.keys())
        return bool(params and params[0] in ("self", "cls"))
    except (ValueError, TypeError):
        return False



"""emotion.core — runtime engine: analysis, registry, profiling, decoration."""

from .decorator import emotion
from .registry  import (
    register, get_data, get_all_names,
    set_description, record_profile,
    record_call_start, record_call_end, record_co_call,
    update_relationships, compute_evolved_emotion, should_evolve,
)
from .analyzer  import extract_features, classify_emotion, is_method
from .profiler  import Profiler, ProfileSnapshot

__all__ = [
    "emotion",
    "register", "get_data", "get_all_names",
    "set_description", "record_profile",
    "record_call_start", "record_call_end", "record_co_call",
    "update_relationships", "compute_evolved_emotion", "should_evolve",
    "extract_features", "classify_emotion", "is_method",
    "Profiler", "ProfileSnapshot",
]



from __future__ import annotations

import time
import tracemalloc
from typing import NamedTuple
tracemalloc.start()


class ProfileSnapshot(NamedTuple):
    wall_time_ms: float
    cpu_time_ms: float 
    memory_delta_kb: float
    memory_peak_kb: float  


class Profiler:
    """Context manager that captures wall time, CPU time, and heap usage."""

    __slots__ = (
        "_start_wall_time",
        "_start_cpu_time",
        "_memory_before_bytes",
        "snapshot",
    )

    def __init__(self) -> None:
        self._start_wall_time: float = 0.0
        self._start_cpu_time: float = 0.0
        self._memory_before_bytes: int = 0
        self.snapshot: ProfileSnapshot | None = None

    def __enter__(self) -> "Profiler":
        tracemalloc.reset_peak()
        self._memory_before_bytes = tracemalloc.get_traced_memory()[0]
        self._start_wall_time = time.perf_counter()
        self._start_cpu_time = time.process_time()
        return self

    def __exit__(self, *_) -> None:
        wall_time_ms = (time.perf_counter() - self._start_wall_time) * 1000.0
        cpu_time_ms = (time.process_time() - self._start_cpu_time) * 1000.0

        memory_current_bytes, memory_peak_bytes = tracemalloc.get_traced_memory()
        memory_delta_bytes = memory_current_bytes - self._memory_before_bytes

        self.snapshot = ProfileSnapshot(
            wall_time_ms=round(wall_time_ms, 2),
            cpu_time_ms=round(cpu_time_ms, 3),
            memory_delta_kb=round(memory_delta_bytes / 1024.0, 2),
            memory_peak_kb=round(memory_peak_bytes / 1024.0, 2),
        )
